Package: popbayes
Type: Package
Version: 1.3
Title: Bayesian Model to Estimate Population Trends from Counts Series
Authors@R: c(
    person(
      given   = "Nicolas",
      family  = "Casajus",
      email   = "nicolas.casajus@fondationbiodiversite.fr",
      role    = c("aut", "cre", "cph"),
      comment = c(ORCID = "0000-0002-5537-5294")),
    person(
      given   = "Roger",
      family  = "Pradel",
      email   = "roger.pradel@cefe.cnrs.fr",
      role    = "aut",
      comment = c(ORCID = "0000-0002-2684-9251")))
Description: Infers the trends of one or several animal populations over time 
    from series of counts. It does so by accounting for count precision 
    (provided or inferred based on expert knowledge, e.g. guesstimates), 
    smoothing the population rate of increase over time, and accounting for the
    maximum demographic potential of species. Inference is carried out in a 
    Bayesian framework. This work is part of the FRB-CESAB working group
    AfroBioDrivers 
    <https://www.fondationbiodiversite.fr/en/the-frb-in-action/programs-and-projects/le-cesab/afrobiodrivers/>.
URL: https://frbcesab.github.io/popbayes/,
        https://github.com/frbcesab/popbayes
BugReports: https://github.com/frbcesab/popbayes/issues
Depends: R (>= 3.5)
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
SystemRequirements: JAGS (>= 4.1)
Imports: graphics, grDevices, R2jags, stats, cli
Suggests: knitr, rmarkdown
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Packaged: 2026-04-20 10:14:25 UTC; nicolas
Author: Nicolas Casajus [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-5537-5294>),
  Roger Pradel [aut] (ORCID: <https://orcid.org/0000-0002-2684-9251>)
Maintainer: Nicolas Casajus <nicolas.casajus@fondationbiodiversite.fr>
Repository: CRAN
Date/Publication: 2026-04-20 10:30:02 UTC
Built: R 4.6.0; ; 2026-04-28 08:38:42 UTC; unix

# Generated by roxygen2: do not edit by hand

export(bugs_to_df)
export(diagnostic)
export(filter_series)
export(fit_trend)
export(format_data)
export(list_series)
export(plot_series)
export(plot_trend)
export(read_bugs)
export(read_series)
export(series_to_df)
export(w_to_rmax)
importFrom(grDevices,dev.off)
importFrom(graphics,abline)
importFrom(graphics,axTicks)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,grid)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,text)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# popbayes <img src="man/figures/logo.png" height="120px" align="right" style="float:right; height:120px;"/>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/popbayes)](https://CRAN.R-project.org/package=popbayes/)
[![R CMD
Check](https://github.com/frbcesab/popbayes/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/frbcesab/popbayes/actions/workflows/R-CMD-check.yaml)
[![Website
deployment](https://github.com/frbcesab/popbayes/actions/workflows/pkgdown.yaml/badge.svg)](https://github.com/frbcesab/popbayes/actions/workflows/pkgdown.yaml)
[![License: GPL (\>=
2)](https://img.shields.io/badge/License-GPL%20%28%3E%3D%202%29-blue.svg)](https://choosealicense.com/licenses/gpl-2.0/)
<!-- badges: end -->

The goal of the R package `popbayes` is to infer trends of one or
several populations over time from series of counts. It does so by
accounting for count precision (provided or inferred based on expert
knowledge, e.g. guesstimates), smoothing the population rate of increase
over time, and accounting for the maximum demographic potential of
species. Inference is carried out in a Bayesian framework. This work is
part of the FRB-CESAB working group
[AfroBioDrivers](https://www.fondationbiodiversite.fr/en/the-frb-in-action/programs-and-projects/le-cesab/afrobiodrivers/).

## Installation

**Before using this package, users need to install the freeware
[JAGS](https://mcmc-jags.sourceforge.io/).**

You can install the stable version of the package from the
[CRAN](https://cran.r-project.org) with:

``` r
install.packages("popbayes")
```

Alternatively you can install the development version from
[GitHub](https://github.com/) with:

``` r
# install.packages("remotes")

remotes::install_github("frbcesab/popbayes", build_vignettes = TRUE)
```

## Overview

![](vignettes/docs/popbayes-diagram.png)

## Get started

Please read the [Get
started](https://frbcesab.github.io/popbayes/articles/popbayes.html)
vignette.

## Citation

Please cite this package as:

> Casajus N. & Pradel R. (2026) popbayes: Bayesian model to estimate
> population trends from counts series. R package version 1.3. URL:
> <https://frbcesab.github.io/popbayes/>.

You can also run:

``` r
citation("popbayes")

## A BibTeX entry for LaTeX users is:
## 
## @Manual{,
##   title  = {{popbayes}: {B}ayesian model to estimate population trends from counts series,
##   author = {{Casajus N.}, and {Pradel R.}},
##   year   = {2026},
##   note   = {R package version 1.3},
##   url    = {https://frbcesab.github.io/popbayes/},
## }
```

#' 
#' Create an Hexagonal Sticker for the Package
#' 

elephant <- png::readPNG(here::here("inst", "elephant.png"))

p <- ggplot2::ggplot(ggplot2::aes(x = mpg, y = wt), data = mtcars) + 
  ggplot2::geom_point(size = 0.1, colour = "white") + 
  ggplot2::geom_smooth(method = "gam", colour = "white", fill = "white",
                       size = 0.2) + 
  rphylopic::add_phylopic(elephant, alpha = 1, x = 28, y = 4.75, ysize = 6.5, 
                          col = "white") +
  ggplot2::theme_void() + 
  ggpubr::theme_transparent()


hexSticker::sticker(
  
  subplot   = p,
  package   = "popbayes",
  filename  = here::here("man", "figures", "hexsticker.png"),
  dpi       = 600,
  
  p_size    = 6.0,         # Title
  u_size    = 1.2,         # URL
  p_family  = "Aller_Rg",
  
  p_color   = "#FFFFFF",   # Title
  h_fill    = "#2C4370",   # Background
  h_color   = "#5C6F94",   # Border
  u_color   = "#FFFFFF",   # URL
  
  p_x       = 1.00,        # Title
  p_y       = 0.65,        # Title
  s_x       = 1.00,        # Subplot
  s_y       = 1.12,        # Subplot
  
  s_width   = 1.40,        # Subplot
  s_height  = 1.10,        # Subplot
  
  url       = "https://frbcesab.github.io/popbayes",
  
  spotlight = TRUE,
  l_alpha   = 0.10,
  l_width   = 4,
  l_height  = 4
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse   = TRUE,
  comment    = "#>",
  fig.width  = 7,
  fig.height = 7,
  out.width  = "100%",
  dpi        = 300
)

## ----'setup', echo = FALSE----------------------------------------------------
library("popbayes")

## ----echo = FALSE, out.width = "100%", fig.cap = "Framework of `popbayes`", fig.align = 'center'----
knitr::include_graphics("docs/popbayes-diagram.png")

## ----'load-data'--------------------------------------------------------------
## Define filename path ----
file_path <- system.file("extdata", "garamba_survey.csv", package = "popbayes")

## Read CSV file ----
garamba <- read.csv(file = file_path)

## ----'print-data', echo = FALSE-----------------------------------------------
knitr::kable(head(garamba, 20), caption = "The Garamba dataset (first 20 rows)", 
             align = c("c", "l", "c", "c", "c", "r", "r", "r", "c", "r", "r"))

## ----'load-data-2'------------------------------------------------------------
data("species_info")

## ----'print-data-2', echo = FALSE---------------------------------------------
knitr::kable(species_info[ , c(3, 6, 7, 8, 9)], 
             caption = "Species with count conversion information in popbayes", 
             align = c("l", "c", "c", "r", "r"))

## ----'create-info'------------------------------------------------------------
## Extract the relevant columns of the package table "species_info" ----
info_from_package <- species_info[ , c("species", "pref_field_method", "conversion_A2G", "rmax")]

## Add the new species ----
new_conversion_info <- data.frame("species"           = c("Taurotragus oryx","Taurotragus derbianus"),
                                  "pref_field_method" = "G",
                                  "conversion_A2G"    = 2.302,
                                  "rmax"              = 0.1500)

## Append the new species ----
info <- rbind(info_from_package, new_conversion_info)
info

## ----'check-location'---------------------------------------------------------
unique(garamba$"location")

sum(is.na(garamba$"location"))   # Are there any missing values?

## ----'check-species'----------------------------------------------------------
unique(garamba$"species")

sum(is.na(garamba$"species"))   # Are there any missing values?

## Are there species absent from the 'species_info' popbayes dataset?
garamba_species <- unique(garamba$"species")
garamba_species[which(!(garamba_species %in% species_info$"species"))]

## ----'check-date'-------------------------------------------------------------
is.numeric(garamba$"date")     # Are dates in a numerical format?

sum(is.na(garamba$"date"))     # Are there any missing values?

range(garamba$"date")          # What is the temporal extent?

## ----'convert-date'-----------------------------------------------------------
## Convert a character to a date object ----
x <- as.Date("2021/05/19")
x

## Convert a date to a numeric (number of days since 1970/01/01) ----
x <- as.numeric(x)
x

## Check ----
as.Date(x, origin = as.Date("1970/01/01"))

## ----'check-counts'-----------------------------------------------------------
is.numeric(garamba$"count")   # Are counts in a numerical format?

range(garamba$"count")        # What is the range of values?

sum(is.na(garamba$"count"))   # Are there any missing values?

## ----'check-stat'-------------------------------------------------------------
unique(garamba$"stat_method")

sum(is.na(garamba$"stat_method"))   # Are there any missing values?

## ----'check-field'------------------------------------------------------------
unique(garamba$"field_method")

sum(is.na(garamba$"field_method"))   # Are there any missing values?

## ----'define-path-1', eval = TRUE, echo = FALSE-------------------------------
path <- tempdir()

## ----'define-path-2', eval = FALSE, echo = TRUE-------------------------------
# path <- "the_folder_to_store_outputs"

## ----'format-data'------------------------------------------------------------
garamba_formatted <- popbayes::format_data(data              = garamba, 
                                           path              = path,
                                           field_method      = "field_method",
                                           pref_field_method = "pref_field_method",
                                           conversion_A2G    = "conversion_A2G",
                                           rmax              = "rmax")

## ----'explore-series'---------------------------------------------------------
## Class of the object ----
class(garamba_formatted)

## Number of elements (i.e. number of count series) ----
length(garamba_formatted)

## Get series names ----
popbayes::list_series(path)

## ----'filter-series'----------------------------------------------------------
## Retrieve series by species and location ----
a_buselaphus <- popbayes::filter_series(data     = garamba_formatted, 
                                        species  = "Alcelaphus buselaphus",
                                        location = "Garamba")

## ----'print-series'-----------------------------------------------------------
print(a_buselaphus)

## ----'plot-series-1', fig.width=12, fig.height=6, out.width="100%"------------
popbayes::plot_series("garamba__alcelaphus_buselaphus", path = path)

## ----'list-folder-1', echo = TRUE, eval = FALSE-------------------------------
# list.files(path, recursive = TRUE)

## ----'list-folder-2', echo = FALSE, eval = TRUE-------------------------------
list.files(path, recursive = TRUE, pattern = "^garamba__")

## ----'read-series', eval = FALSE----------------------------------------------
# a_buselaphus <- popbayes::read_series("garamba__alcelaphus_buselaphus", path = path)

## ----'run-jags', eval = FALSE-------------------------------------------------
# a_buselaphus_bugs <- popbayes::fit_trend(a_buselaphus, path = path)

## ----'read-bugs', eval = FALSE------------------------------------------------
# a_buselaphus_bugs <- popbayes::read_bugs("garamba__alcelaphus_buselaphus", path = path)

## ----'diagnostic', eval = FALSE-----------------------------------------------
# popbayes::diagnostic(a_buselaphus_bugs)
# #> All models have converged.

## ----'re-run-jags', eval = FALSE----------------------------------------------
# a_buselaphus_bugs <- popbayes::fit_trend(a_buselaphus, path = path, ni = 100000, nb = 20000)

## ----'plot-trend-1', fig.width=12, fig.height=6, out.width="100%", eval = FALSE, echo = TRUE----
# popbayes::plot_trend("garamba__alcelaphus_buselaphus", path = path)

## ----echo = FALSE, out.width = "100%", fig.align = 'center'-------------------
knitr::include_graphics("docs/garamba__alcelaphus_buselaphus.png")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>"
)

## ----'rmax'-------------------------------------------------------------------
species_name <- c("Impala", "Tiang", "Blue wildebeest", "Roan", "Buffalo", 
                  "Eland", "Giraffe", "Elephant")

adult_female_body_mass <- c(55, 127, 230, 250, 400, 450, 702, 2873)

species <- data.frame(adult_female_body_mass, row.names = species_name)

species$rmax <- 1.375 * adult_female_body_mass ^ (-0.315)
species["Eland", "rmax"]    <- 0.150                 # Sinclair (1996)
species["Elephant", "rmax"] <- 0.07                  # Foley & Faust (2010)
species["Giraffe", "rmax"]  <- 0.125                 # Suraud et al. (2012)

species

## ----'conversion_fact'--------------------------------------------------------
categories <- c("Medium light and brown species (20-150kg)",
                "Large light and brown species (>150kg)",
                "Large dark (>150kg)", "Giraffe", "Elephant")

short_names        <- c("MLB", "LLB", "LD", "Giraffe", "Elephant")
conversion_facts   <- c(6.747, 2.302, 0.561, 3.011, 0.659)
pref_field_methods <- c("G", "G", "A", "A", "A")

category_info <- data.frame("category"          = categories,
                            "acronym"           = short_names,
                            "conversion_fact"   = conversion_facts, 
                            "pref_field_method" = pref_field_methods)
category_info

## -----------------------------------------------------------------------------
species$"category" <- c("MLB", "MLB", "LLB", "LLB", "LD", "LLB", "Giraffe", 
                        "Elephant")
species

#' @keywords internal
"_PACKAGE"

# Imports: start ---- 
#' @importFrom grDevices dev.off
#' @importFrom graphics abline axTicks axis box grid lines mtext par points 
#' @importFrom graphics polygon text
# Imports: end ----

NULL

#' Species information dataset
#' 
#' @description
#' This dataset contains information about 15 African mammal species.
#' It can be used in the function `format_data()` to convert individual counts 
#' estimated from a field method to a preferred field method. The field method 
#' can be `A` (aerial counts) or `G` (ground counts). See [format_data()] for 
#' further information. It also contains the maximum population growth rate 
#' (i.e. the maximum change in log population size).
#' 
#' User can take this dataset as a template to add information for missing 
#' species. Note that only `species`, `pref_field_method`, `conversion_A2G`, 
#' and `rmax` are required.
#'   
#' @format A `data.frame` with 15 rows (African mammals species) and the 
#' following variables:
#' \describe{
#'   \item{order}{the order of the species}
#'   \item{family}{the family of the species}
#'   \item{species}{the species binomial name}
#'   \item{english}{the species English name}
#'   \item{french}{the species French name}
#'   \item{category}{the detectability category of the species. One of 
#'     \code{MLB} for Medium-sized Light and Brown species (20-150kg),
#'     \code{LLB} for Large Light and Brown species (>150kg),
#'     \code{LD} for Large Dark (>150kg), \code{Elephant}, and \code{Giraffe}}
#'   \item{pref_field_method}{the preferred field method of the species. One of 
#'     \code{A} for Aerial counts, and \code{G} for Ground counts}
#'   \item{conversion_A2G}{the conversion multiplicative factor (corresponding  
#'     to the detectability category) used to convert aerial to ground counts}
#'   \item{rmax}{the maximum population growth rate}
#' }
#' 
#' @examples 
#' data("species_info")
#' species_info

"species_info"


#' African mammals survey in the Garamba National Park
#' 
#' @description
#' This dataset contains individual counts of 10 African mammal species
#' in the Garamba National Park (Democratic Republic of the Congo) from 1976 to
#' 2017.
#'   
#' @format A `data.frame` with 141 rows (counts) and the following 8 variables:
#' \describe{
#'   \item{location}{the location of the survey (Garamba)}
#'   \item{species}{the binomial name of the species}
#'   \item{date}{the date of the survey}
#'   \item{stat_method}{the method used to estimate individuals counts. One of
#'     \code{T} (total counts), \code{G} (guesstimate), and \code{S} 
#'     (sampling counts)}
#'   \item{field_method}{the field method used to collect data. One of 
#'     \code{A} (aerial counts), and \code{G} (ground counts)}
#'   \item{count}{number of individuals}
#'   \item{lower_ci}{lower boundary of the 95% confidence interval of the counts
#'     (only for sampling counts)}
#'   \item{upper_ci}{upper boundary of the 95% confidence interval of the counts
#'     (only for sampling counts)}
#'   \item{pref_field_method}{the preferred field method of the species. One of 
#'     \code{A} for Aerial counts, and \code{G} for Ground counts}
#'   \item{conversion_A2G}{the conversion multiplicative factor (corresponding  
#'     to the detectability category) used to convert aerial to ground counts}
#'   \item{rmax}{the maximum population growth rate}
#' }
#' 
#' @references 
#' Hillman Smith K & Kalpers J (2015) _Garamba Conservation in Peace & War_. 
#' Hillman Smith publisher, 448pp. ISBN: 9789966185105.\cr
#' Monico M (2014) 
#' _Aerial Survey Report March 2014 - Garamba National Park, DRC_. 
#' African Parks Network/ICCN/Pan-African Elephant Aerial Survey, 42pp.\cr
#' Spies K _et al._ (2017) 
#' _Aerial Survey Report April 2017 - Garamba National Park, DRC_.
#' African Parks Network/EU/ICCN. 38pp.
#' 
#' @examples 
#' data("garamba")
#' head(garamba, 30)

"garamba"

#' Compute rmax from adult female body mass
#' 
#' @description
#' The demographic potential of a species is limited. The intrinsic rate of 
#' increase `rmax` is the maximum increase in log population size that a 
#' species can attain in a year. According to Sinclair (2003), it is related 
#' to the body mass of adult females by: \eqn{1.375 \times W^{-0.315}}
#' 
#' @param w a numerical vector. Adult female body mass (in kg).
#' 
#' @export
#'  
#' @return A numerical vector of `rmax` values.
#' 
#' @references 
#' Sinclair (2013) Mammal population regulation, keystone processes and
#'   ecosystem dynamics. _Philosophical Transactions: Biological Sciences_,
#'   **358**, 1729-1740.
#' 
#' @examples
#' ## Set adult female body mass ----
#' body_masses <- c(55, 127)
#' 
#' ## Add species names ----
#' names(body_masses) <- c("Impala", "Tiang")
#' 
#' ## Compute species rmax ----
#' w_to_rmax(body_masses)

w_to_rmax <- function(w) 1.375 * w ^ -0.315

#' Plot original vs converted counts
#' 
#' @description
#' This function plots a panel of two graphics for one count series 
#' (previously generated by [format_data()]):
#' * on the left side, scatter plot overlapping original (black points) and 
#'   converted counts (grey points);
#' * on the right side, scatter plot of converted counts with boundaries of the
#'   95% confident interval.
#' 
#' @param series a `character` string. The count series names (can be 
#'   retrieved by running [list_series()]).
#'   
#' @param title a `logical`. If `TRUE` (default) a title (series name) is 
#'   added.
#'   
#' @param path a `character` string. The directory in which count series have 
#'   been saved by the function [format_data()].
#'   
#' @param path_fig a `character` string. The directory where to save the plot 
#'   (if `save = TRUE`). This directory must exist and can be an absolute or a 
#'   relative path.
#'   
#' @param save a `logical`. If `TRUE` (default is `FALSE`) the plot is saved in 
#'   `path_fig`.
#'
#' @return No return value.
#' 
#' @export
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#' 
#' ## Get series names ----
#' popbayes::list_series(path = temp_path)
#' 
#' ## Plot for Alcelaphus buselaphus at Garamba ----
#' popbayes::plot_series("garamba__alcelaphus_buselaphus", path = temp_path)

plot_series <- function(series, title = TRUE, path = ".", path_fig = ".",
                        save = FALSE) {
  
  
  if (!dir.exists(path)) {
    stop("The directory '", path, "' does not exist.")
  }
  
  if (missing(series)) {
    stop("Argument 'series' is required.")
  }
  
  if (!is.character(series) || length(series) != 1) {
    stop("Argument 'series' must be character string (one series name).")
  }
  
  if (!dir.exists(file.path(path, series))) {
    stop("The directory '", file.path(path, series), "' does not exist.")
  }
  
  if (!file.exists(file.path(path, series, paste0(series, "_data.RData")))) {
    stop("The file '", file.path(path, series, 
                                 paste0(series, "_data.RData")), 
         "' does not exist. Did you forget to run format_data()?")
  }
  
  
  ## Load formatted data ----
  
  data <- get(load(file.path(path, series, paste0(series, "_data.RData"))))
  
  data_copy <- data
  data <- data[[series]]
  
  
  ## Get plot extent ----
  
  counts_range <- c(0, max(c(data$"data_converted"$"upper_ci_conv", 
                             data$"data_original"$"count_orig")))
  dates_range <- range(data$"date")
  
  
  ## Export as PNG (if required) ----
  
  if (save) {
    
    if (!dir.exists(path_fig)) {
      stop("The directory '", path_fig, "' does not exist.")
    }
    
    filename <- paste0(names(data_copy[series]), "_data.png")
    grDevices::png(filename = file.path(path_fig, filename), width = 12, 
                   height = 6, pointsize = 14, res = 600, units = "in")
  }
  
  
  ## Graphical parameters ----
  
  opar <- par(no.readonly = TRUE)
  on.exit(par(opar))
  
  par(family = "serif", mgp = c(1.5, 0.1, 0), mar = c(1.5, 3.1, 1.9, 1.1),
      cex.axis = 0.75)
  par(mfrow = c(1, 2))
  
  
  ###
  ### Left plot - Original vs. Converted
  ###
  
  ## Empty plot ----
  
  plot(0, type = "n", xlim = dates_range, ylim = counts_range, axes = FALSE,
       xlab = "", ylab = "Original (black) and converted (grey) counts",
       font.lab = 1)
  
  
  ## Background ----
  
  grid(lwd = 0.25, lty = 1)
  axis(1, at = axTicks(1), lwd = 0)
  axis(2, at = axTicks(2), lwd = 0)
  box(lwd = 1.0, col = "white")
  box(lwd = 0.5)
  
  
  ## Deviation between original and converted data ----
  
  for (i in seq_len(nrow(data$"data_original"))) {
    
    lines(x = rep(data$"data_original"[i, "date"], 2),
          y = c(data$"data_original"[i, "count_orig"], 
                data$"data_converted"[i, "count_conv"]),
          lty = 4, lwd = 0.75)
  }
  
  
  ## Add data ----
  
  points(x = data$"data_original"$"date", 
         y = data$"data_original"$"count_orig", pch = 19, cex = 1.2)
  
  points(x = data$"data_converted"$"date", 
         y = data$"data_converted"$"count_conv", pch = 19, cex = 1.2)
  
  points(x = data$"data_converted"$"date", 
         y = data$"data_converted"$"count_conv", pch = 19, col = "#aaaaaa")
  
  
  ###
  ### Right plot - Converted with 95% CI
  ###
  
  
  ## Empty plot ----
  
  plot(0, type = "n", xlim = dates_range, ylim = counts_range, axes = FALSE,
       xlab = "", ylab = "Converted counts with 95% CI",
       font.lab = 1)
  
  
  ## Background ----
  
  grid(lwd = 0.25, lty = 1)
  axis(1, at = axTicks(1), lwd = 0)
  axis(2, at = axTicks(2), lwd = 0)
  box(lwd = 1.0, col = "white")
  box(lwd = 0.5)
  
  
  ## Add 95% CI ----
  
  for (i in seq_len(nrow(data$"data_converted"))) {
    
    lines(x = rep(data$"data_converted"[i, "date"], 2),
          y = c(data$"data_converted"[i, "lower_ci_conv"], 
                data$"data_converted"[i, "upper_ci_conv"]))
    
    
    lines(x = c(data$"data_converted"[i, "date"] - 
                  data$"data_converted"[i, "date"] * 0.0002,
                data$"data_converted"[i, "date"] + 
                  data$"data_converted"[i, "date"] * 0.0002),
          y = rep(data$"data_converted"[i, "lower_ci_conv"], 2))
    
    lines(x = c(data$"data_converted"[i, "date"] - 
                  data$"data_converted"[i, "date"] * 0.0002,
                data$"data_converted"[i, "date"] + 
                  data$"data_converted"[i, "date"] * 0.0002),
          y = rep(data$"data_converted"[i, "upper_ci_conv"], 2))
  }
  
  
  ## Add data ----
  
  points(x = data$"data_converted"$"date", 
         y = data$"data_converted"$"count_conv", pch = 19, cex = 1.2)
  
  points(x = data$"data_converted"$"date", 
         y = data$"data_converted"$"count_conv", pch = 19, col = "#aaaaaa")
  
  
  ## Add title ----
  
  if (title)
    mtext(paste(data$"location", "-", data$"species"), side = 3, line = 0.5, 
          adj = 1, at = par()$usr[2], cex = 1.2, font = 4)
  
  
  ## Restore user graphical parameters ----
  
  if (save)
    dev.off()
  
  invisible(NULL) 
}



#' Plot estimated population trend
#' 
#' @description
#' This function plots a panel of two graphics for one BUGS model
#' (previously generated by [fit_trend()]):
#' * on the left side, the population trend estimated by the Bayesian model 
#'   (blue line) with the 95% CI (gray envelop). Dots (with intervals) 
#'   represent converted counts passed to the model (with the 95% CI);
#' * on the right side, a bar plot of estimated relative growth rates (r) by 
#'   date. Dark bars are real estimated r.
#' 
#' @param series a `character` string. The count series name (can be 
#'   retrieved by running [list_series()]).
#'   
#' @param title a `logical`. If `TRUE` (default) a title (series name) is 
#'   added.
#'   
#' @param path a `character` string. The directory in which count series (and 
#'   BUGS outputs) have been saved by the function [format_data()] (and by
#'   [fit_trend()]).
#'   
#' @param path_fig a `character` string. The directory where to save the plot 
#'   (if `save = TRUE`). This directory must exist and can be an absolute or a 
#'   relative path.
#'   
#' @param save a `logical`. If `TRUE` (default is `FALSE`) the plot is saved in 
#'   `path_fig`.
#'
#' @return No return value.
#' 
#' @export
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#' 
#' ## Select one serie ----
#' a_buselaphus <- popbayes::filter_series(garamba_formatted, 
#'                                         location = "Garamba",
#'                                         species  = "Alcelaphus buselaphus")
#' \donttest{
#' ## Fit population trends (requires JAGS) ----
#' a_buselaphus_mod <- popbayes::fit_trend(a_buselaphus, path = temp_path)
#' 
#' ## Plot estimated population trend ----
#' popbayes::plot_trend(series = "garamba__alcelaphus_buselaphus", 
#'                      path   = temp_path)
#' }

plot_trend <- function(series, title = TRUE, path = ".", path_fig = ".", 
                       save = FALSE) {
  
  if (!dir.exists(path)) {
    stop("The directory '", path, "' does not exist.")
  }
  
  if (missing(series)) {
    stop("Argument 'series' is required.")
  }
  
  if (!is.character(series) || length(series) != 1) {
    stop("Argument 'series' must be a character string (one series name).")
  }
  
  if (!dir.exists(file.path(path, series))) {
    stop("The directory '", file.path(path, series), "' does not exist.")
  }
  
  if (!file.exists(file.path(path, series, paste0(series, "_data.RData")))) {
    stop("The file '", file.path(path, series, 
                                 paste0(series, "_data.RData")), 
         "' does not exist. Did you forget to run format_data()?")
  }
  
  if (!file.exists(file.path(path, series, paste0(series, "_bugs.RData")))) {
    stop("The file '", file.path(path, series, 
                                 paste0(series, "_bugs.RData")), 
         "' does not exist. Did you forgot to run fit_trend()?")
  }
  
  if (!is.logical(title)) {
    stop("Argument 'title' must be TRUE or FALSE.")
  }
  
  if (!is.logical(save)) {
    stop("Argument 'save' must be TRUE or FALSE.")
  }
  
  
  ## Load formatted data ----
  
  data <- get(load(file.path(path, series, paste0(series, "_data.RData"))))
  
  data_copy <- data
  
  data <- data[[1]]$"data_converted"
  
  
  ## Load BUGS outputs ----
  
  outputs <- get(load(file.path(path, series, paste0(series, "_bugs.RData"))))
  outputs <- outputs[[1]]$"BUGSoutput"$"summary"
  
  
  ## Select estimated N ----
  
  outputs_n <- outputs[grep("^N\\[", rownames(outputs)), c("mean", "2.5%", 
                                                           "97.5%")]
  rownames(outputs_n) <- NULL
  colnames(outputs_n) <- c("count_jags", "lower_ci_jags", "upper_ci_jags")
  
  
  ## Create table N for graphic ----
  
  data_n_for_graph <- data.frame("date"     = data$"date", 
                                 "count"    = data$"count_conv",
                                 "lower_ci" = data$"lower_ci_conv", 
                                 "upper_ci" = data$"upper_ci_conv")
  
  data_n_for_graph <- cbind(data_n_for_graph, outputs_n)
  
  
  ## Select estimated r ----
  
  outputs_r <- outputs[grep("^r\\[", rownames(outputs)), "mean"]
  outputs_r <- c(outputs_r, NA)
  outputs_r <- data.frame("date" = data$"date", "r_jags" = outputs_r)
  rownames(outputs_r) <- NULL
  
  
  ## Create table r for graphic ----
  
  dates_seq <- data.frame(
    "date" = seq(min(outputs_r$"date"), max(outputs_r$"date")))
  
  outputs_r <- merge(outputs_r, dates_seq, by = "date", all = TRUE)
  
  outputs_r$"color" <- ifelse(is.na(outputs_r$"r_jags"), "#888888", "black")
  outputs_r$"lwd"   <- ifelse(is.na(outputs_r$"r_jags"), 1, 1.2)
  
  for (i in seq_len(nrow(outputs_r))) {
    if (is.na(outputs_r[i, "r_jags"])) {
      outputs_r[i, "r_jags"] <- outputs_r[i - 1, "r_jags"]
    }
  }
  
  data_r_for_graph <- outputs_r
  
  
  ## Get plot extent ----
  
  dates_range  <- range(data_n_for_graph$"date")
  
  counts_range <- c(0, max(c(data_n_for_graph$"upper_ci", 
                             data_n_for_graph$"upper_ci_jags")))
  
  growth_range <- c(min(data_r_for_graph$"r_jags"),
                    max(data_r_for_graph$"r_jags"))
  
  growth_range[1] <- ifelse(growth_range[1] > 0, 0, growth_range[1])
  growth_range[2] <- ifelse(growth_range[2] < 0, 0, growth_range[2])
  growth_range[2] <- growth_range[2] + 
    (growth_range[2] - growth_range[1]) * 0.05
  
  
  ## Export as PNG (if required) ----
  
  if (save) {
    
    if (!dir.exists(path_fig)) {
      stop("The directory '", path_fig, "' does not exist.")
    }
    
    filename <- paste0(names(data_copy), "_trend.png")
    grDevices::png(filename = file.path(path_fig, filename), width = 12, 
                   height = 6, pointsize = 14, res = 600, units = "in")
  }
  
  
  ## Graphical parameters ----
  
  opar <- par(no.readonly = TRUE)
  on.exit(par(opar))
  
  par(family = "serif", mgp = c(1.5, 0.1, 0), mar = c(1.5, 3.1, 1.9, 1.1),
      cex.axis = 0.75)
  par(mfrow = c(1, 2))
  
  
  
  ###
  ### Left plot - Population trend
  ###
  
  
  
  ## Empty plot ----
  
  plot(0, type = "n", xlim = dates_range, ylim = counts_range, axes = FALSE,
       xlab = "", ylab = "Population size", font.lab = 1)
  
  
  ## Background ----
  
  grid(lwd = 0.25, lty = 1)
  axis(1, at = axTicks(1), lwd = 0)
  axis(2, at = axTicks(2), lwd = 0)
  box(lwd = 1.0, col = "white")
  box(lwd = 0.5)
  
  
  ## Add JAGS CI polygon ----
  
  polygon(x = c(data_n_for_graph$"date", 
                data_n_for_graph$"date"[rev(seq_len(nrow(data_n_for_graph)))]),
          y = c(data_n_for_graph[ , "lower_ci_jags"],
                data_n_for_graph[rev(seq_len(nrow(data_n_for_graph))), 
                                 "upper_ci_jags"]),
          col = "#bbbbbb", border = "#bbbbbb", lwd = 1)
  
  
  ## Add JAGS mean prediction ----
  
  lines(x = data_n_for_graph$"date", y = data_n_for_graph$"count_jags", 
        col = "steelblue", lwd = 2)
  
  
  ## Add Observed (converted) 95% CI ----
  
  for (i in seq_len(nrow(data_n_for_graph))) {
    
    lines(x = rep(data_n_for_graph[i, "date"], 2),
          y = c(data_n_for_graph[i, "lower_ci"], 
                data_n_for_graph[i, "upper_ci"]))
    
    
    lines(x = c(data_n_for_graph[i, "date"] - 
                  data_n_for_graph[i, "date"] * 0.0002,
                data_n_for_graph[i, "date"] + 
                  data_n_for_graph[i, "date"] * 0.0002),
          y = rep(data_n_for_graph[i, "lower_ci"], 2))
    
    lines(x = c(data_n_for_graph[i, "date"] - 
                  data_n_for_graph[i, "date"] * 0.0002,
                data_n_for_graph[i, "date"] + 
                  data_n_for_graph[i, "date"] * 0.0002),
          y = rep(data_n_for_graph[i, "upper_ci"], 2))
  }
  
  
  ## Add Observed (converted) counts ----
  
  points(x = data_n_for_graph$"date", y = data_n_for_graph$"count", pch = 19, 
         cex = 1.2)
  
  
  
  ###
  ### Right plot - Estimated r barplot
  ###
  
  
  
  ## Empty plot ----
  
  plot(0, type = "n", xlim = dates_range, ylim = growth_range, axes = FALSE,
       xlab = "", ylab = "Relative growth rate (r)",
       font.lab = 1)
  
  
  ## Background ----
  
  grid(lwd = 0.25, lty = 1)
  axis(1, at = axTicks(1), lwd = 0)
  axis(2, at = axTicks(2), lwd = 0)
  
  
  ## Add data ----
  
  points(x = data_r_for_graph$"date", y = data_r_for_graph$"r", type = "h", 
         col = data_r_for_graph$"color", lwd = data_r_for_graph$"lwd")
  
  
  ## Add average value (top-left text) ----
  
  r_mean <- outputs[grep("^meanr$", rownames(outputs)), c("mean", "2.5%", 
                                                          "97.5%")]
  r_mean <- round(r_mean, 3)
  
  text(x = par()$usr[1], y = growth_range[2],
       labels = bquote(bar(r) == .(r_mean["mean"]) ~ .(paste0("[", 
                                                              r_mean["2.5%"], 
                                                              ", ", 
                                                              r_mean["97.5%"], 
                                                              "]"))), pos = 4)
  
  
  ## Add y = 0 line ----
  
  abline(h = 0, lwd = 1.2, col = "white")
  abline(h = 0, lwd = 0.5)
  box(lwd = 1.0, col = "white")
  box(lwd = 0.5)
  
  
  ## Add title ----
  
  if (title)
    mtext(paste(data_copy[[1]]$"location", "-", data_copy[[1]]$"species"), 
          side = 3, line = 0.5, adj = 1, at = par()$usr[2], cex = 1.2, 
          font = 4)
  
  
  ## Restore user graphical parameters ----
  
  if (save)
    dev.off()
  
  invisible(NULL) 
}

#' Detect individual count series and create an unique identifier
#'
#' For internal use only.
#'
#' @param data a `data.frame`. 
#' 
#' @param quiet a `logical`. If `TRUE`, suppress messages.
#'
#' @return A `data.frame` with three columns:
#'   - `id` (a series unique identifier);
#'   - `location` (the series site);
#'   - `species` (the series species).
#' 
#' @noRd

get_series <- function(data, quiet = TRUE) {
  
  series_id <- unique(paste(data[ , "location"], data[ , "species"], 
                            sep = "__"))
  
  series_infos <- data.frame("id" = series_id)
  
  series_infos$"location" <- unlist(lapply(strsplit(series_id, "__"), 
                                           function(x) x[1]))
  
  series_infos$"species"  <- unlist(lapply(strsplit(series_id, "__"), 
                                           function(x) x[2]))
  
  series_infos <- series_infos[order(series_infos$"id", decreasing = FALSE), ]
  
  series_infos$"id" <- tolower(series_infos$"id")
  series_infos$"id" <- gsub("\\s{2,}", "", series_infos$"id")
  series_infos$"id" <- gsub("\\s", "_", series_infos$"id")
  series_infos$"id" <- gsub("[[:punct:]]", "_", series_infos$"id")
  
  if (!quiet)
    
    cli::cli_alert_success(c(
      "Detecting {.val {nrow(series_infos)}} series with ",
      "{.val {length(unique(series_infos$location))}} location{?s}  and ",
      "{.val {length(unique(series_infos$species))}} species."
    ))
  
  series_infos
}



#' Extract the count series corresponding to a location and/or a species
#' 
#' @description
#' This function identifies the count series relative to a species and/or
#' a location in a named list like the output of function [format_data()]. If 
#' both species and location are provided, the series of counts of the species 
#' at the specified location is extracted. Otherwise, all series corresponding
#' to the specified criterion (species or location) are extracted.
#' 
#' @param data a named `list`. The output of function [format_data()].
#' 
#' @param species a `character` string. A species name.
#' 
#' @param location a `character` string. A site name.
#'
#' @return A subset of `data`, i.e. a named `list`.
#' 
#' @export
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#' 
#' ## Number of count series ----
#' length(garamba_formatted)
#' 
#' ## Retrieve count series names ----
#' popbayes::list_series(path = temp_path)
#' 
#' ## Get data for Alcelaphus buselaphus (at all sites) ----
#' x <- popbayes::filter_series(garamba_formatted, 
#'                              species = "Alcelaphus buselaphus")
#' 
#' ## Get data at Garamba (for all species) ----
#' x <- popbayes::filter_series(garamba_formatted, 
#'                              location = "Garamba")
#' 
#' ## Get data for Alcelaphus buselaphus at Garamba only ----
#' x <- popbayes::filter_series(garamba_formatted, 
#'                              location = "Garamba",
#'                              species  = "Alcelaphus buselaphus")

filter_series <- function(data, species = NULL, location = NULL) {
  
  if (!is.list(data)) {
    stop("Argument 'data' must be an output of format_data().")
  }
  
  if (!("data_converted" %in% names(data[[1]]))) {
    stop("Argument 'data' must be an output of format_data().")
  }
  
  
  ## No filter methods ----
  
  if (is.null(species) && is.null(location)) {
    
    cli::cli_alert_danger(
      "No species nor location provided to filter series"
    )
    
    return(NULL)
  }
  
  
  ## Find series by species ----
  
  if (!is.null(species)) {
    
    if (!is.character(species)) {
      stop("Argument 'species' must be a character string.")
    }
    
    if (length(species) > 1) {
      stop("Argument 'species' must be a character string.")
    }
    
    species_detected <- unlist(lapply(data, function(x, species) 
      ifelse(x$"species" == species, TRUE, FALSE), species = species))
    
    if (sum(species_detected) == 0) {
      stop("Wrong species spelling.")
    }
  }
  
  
  ## Find series by locations ----
  
  if (!is.null(location)) {
    
    if (!is.character(location)) {
      stop("Argument 'location' must be a character string.")
    }
    
    if (length(location) > 1) {
      stop("Argument 'location' must be a character string.")
    }
    
    location_detected <- unlist(lapply(data, function(x, location) 
      ifelse(x$"location" == location, TRUE, FALSE), location = location))
    
    if (sum(location_detected) == 0) {
      stop("Wrong location spelling.")
    }
  }
  
  
  ## Find intersection ----
  
  if (!is.null(species) && !is.null(location)) {
    
    series_match <- which(species_detected & location_detected)
    
    if (length(series_match)) {
      
      cli::cli_alert_success(c(
        "Found {.val {length(series_match)}} series with ",
        "{.val {species}} and {.val {location}}."
      ))
    } else {
      
      cli::cli_alert_danger(
        "No series found with {.val {species}} and {.val {location}}."
      )
      
      return(NULL)
    }
  }
  
  
  ## Otherwise ----
  
  if (!is.null(species) && is.null(location)) {
    
    series_match <- species_detected
    
    cli::cli_alert_success(
      "Found {.val {sum(species_detected)}} series with {.val {species}}."
    )
  }
  
  if (is.null(species) && !is.null(location)) {
    
    series_match <- location_detected
    
    cli::cli_alert_success(
      "Found {.val {sum(location_detected)}} series with {.val {location}}."
    )
  }
  
  data[series_match]
}



#' Extract original/converted count series data from a list
#'
#' @description
#' From the output of the function [format_data()] (or [filter_series()]), this
#' function extracts `data.frame` containing converted counts 
#' (`converted = TRUE`) or original counts (`converted = FALSE`) for one, 
#' several, or all count series.
#' 
#' The resulting `data.frame` has no particular use in `popbayes` but it can be
#' useful for users.
#'
#' @param data a named `list`. The output of [format_data()] or 
#'   [filter_series()].
#' 
#' @param converted a `logical`. If `TRUE` (default) extracts converted counts,
#'   otherwise returns original counts.
#'
#' @return A `data.frame`.
#' 
#' @export
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#' 
#' ## Extract converted count data ----
#' converted_data <- popbayes::series_to_df(garamba_formatted, 
#'                                          converted = TRUE)
#' 
#' ## Extract original count data ----
#' original_data <- popbayes::series_to_df(garamba_formatted, 
#'                                         converted = FALSE)
#' 
#' dim(converted_data)
#' dim(original_data)
#' dim(garamba)

series_to_df <- function(data, converted = TRUE) {
  
  if (!is.list(data)) {
    stop("Argument 'data' must be an output of format_data().")
  }
  
  if (!("data_converted" %in% names(data[[1]]))) {
    stop("Argument 'data' must be an output of format_data().")
  }
  
  if (!is.logical(converted)) {
    stop("Argument 'converted' must be TRUE or FALSE.")
  }
  
  if (length(converted) != 1) {
    stop("Argument 'converted' must be TRUE or FALSE.")
  }
  
  element <- ifelse(converted, "data_converted", "data_original")
  
  data <- lapply(data, function(x) x[[element]])
  data <- do.call(rbind.data.frame, data)
  rownames(data) <- NULL
  
  data
}



#' Import a list of count series previously exported
#'
#' @description 
#' This function imports a list of count series data previously exported by 
#' [format_data()]. Users can import one, several, or all count series data.
#'
#' @param series a vector of `character` strings. One or several count series
#'   names to be imported. If `NULL` (default), all available count series 
#'   will be imported.
#'    
#' @param path a `character` string. The directory in which count series have
#'   been saved by the function [format_data()].
#'
#' @return An n-element `list` (where `n` is the number of count series). See
#'   [format_data()] for further information.
#' 
#' @export
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#' 
#' ## Import all count series ----
#' count_series <- popbayes::read_series(path = temp_path)
#' 
#' ## Import one count series ----
#' a_bus <- popbayes::read_series(series = "garamba__alcelaphus_buselaphus",
#'                                path   = temp_path)

read_series <- function(series = NULL, path = ".") {
  
  
  if (!dir.exists(path)) {
    stop("The directory '", path, "' does not exist.")
  }
  
  filenames <- list.files(path, recursive = TRUE, pattern = "_data\\.RData")
  
  if (length(filenames) == 0) {
    stop("No count series can be found.")  
  }
  
  
  ## All available series names ----
  
  series_names <- strsplit(filenames, .Platform$"file.sep")
  series_names <- unlist(lapply(series_names, function(x) x[length(x)]))
  series_names <- gsub("_data\\.RData", "", series_names)

  
  if (!is.null(series)) {
    
    if (!is.character(series)) {
      stop("Argument 'series' must be a character (series name(s)).")
    }
    
    if (any(!(series %in% series_names))) {
      stop("Some count series cannot be found.")
    } 
    
    series_names <- series
  }

  data_series <- list()
  
  for (series in series_names) {
    
    data_series <- c(data_series, get(load(file.path(path, series, 
                                                     paste0(series, 
                                                            "_data.RData")))))
  }
  
  data_series
}



#' Retrieve the count series names
#'
#' @description This function retrieves the count series names generated by
#' the function [format_data()].
#'
#' @param path a `character` string. The directory in which count series have 
#'   been saved by the function [format_data()].
#'
#' @return A vector of count series names (`character` strings).
#' 
#' @export
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#' 
#' ## Retrieve count series names ----
#' popbayes::list_series(path = temp_path)

list_series <- function(path = ".") {
  
  if (!dir.exists(path)) {
    stop("The directory '", path, "' does not exist.")
  }
  
  filenames <- list.files(path, recursive = TRUE, pattern = "_data\\.RData")
  
  if (length(filenames) == 0) {
    stop("No count series can be found.")  
  }
  
  
  ## All available series names ----
  
  series_names <- strsplit(filenames, .Platform$"file.sep")
  series_names <- unlist(lapply(series_names, function(x) x[length(x)]))
  series_names <- gsub("_data\\.RData", "", series_names)
  
  series_names
}

#' Format count series
#'
#' @description
#' This function provides an easy way to get count series ready to be analyzed 
#' by the package `popbayes`. It must be used prior to all other functions.
#' 
#' This function formats the count series (passed through the argument 
#' `data`) by selecting and renaming columns, checking columns format and 
#' content, and removing missing data (if `na_rm = TRUE`). It converts the 
#' original data frame into a list of count series that will be analyzed later
#' by the function [fit_trend()] to estimate population trends.
#' 
#' To be usable for the estimation of population trends, counts must be 
#' accompanied by information on precision. The population trend model requires 
#' a 95% confident interval (CI).
#' If estimates are total counts or guesstimates, this function will construct 
#' boundaries of the 95% CI by applying the rules set out in 
#' \url{https://frbcesab.github.io/popbayes/articles/popbayes.html}.
#' If counts were estimated by a sampling method the user needs to specify a 
#' measure of precision. Precision is preferably provided in the form of a 95% 
#' CI by means of two fields: `lower_ci` and `upper_ci`. It may also be given 
#' in the form of a standard deviation (`sd`), a variance (`var`), or a 
#' coefficient of variation (`cv`). If the fields `lower_ci` and `upper_ci` are 
#' both absent (or `NA`), fields `sd`, `var`, and `cv` are examined in this 
#' order. When one is found valid (no missing value), a 95% CI is derived 
#' assuming a normal distribution. 
#' The field `stat_method` must be present in `data` to indicate
#' if counts are **total counts** (`'T'`), **sampling** (`'S'`), or 
#' **guesstimate** (`'X'`).
#' 
#' If a series mixes aerial and ground counts, a field `field_method` must 
#' also be present and must contain either `'A'` (aerial counts), or `'G'` 
#' (ground counts). As all counts must eventually refer to the same field 
#' method for a correct estimation of trend, a conversion will be performed to  
#' homogenize counts. This conversion is based on a **preferred field method**
#' and a **conversion factor** both specific to a species/category. 
#' The preferred field method specifies the conversion direction. The 
#' conversion factor is the multiplicative factor that must be applied to an 
#' aerial count to get an equivalent ground count (note that if the preferred 
#' field method is `'A'`, ground counts will be divided by the conversion 
#' factor to get the equivalent aerial count).
#' 
#' The argument `rmax` represents the maximum change in log population size 
#' between two dates (i.e. the relative rate of increase). It will be used 
#' by [fit_trend()] but must be provided in this function.
#' 
#' These three parameters, named `pref_field_method`, `conversion_A2G`, and 
#' `rmax` can be present in `data` or in a second `data.frame` 
#' (passed through the argument `info`).
#' Alternatively, the package `popbayes` provides their values for some 
#' African large mammals.
#' 
#' **Note:** If the field `field_method` is absent in `data`, counts are 
#' assumed to be obtained with one field method.
#'
#'
#'
#' @param data a `data.frame` with at least five columns: `location`, 
#'   `species`, `date`, `count`, and `stat_method`.
#'   
#'   The `stat_method` field indicates the method used to estimate counts. It 
#'   can contain: `T` (total counts), `X` (guesstimate), and/or `S` (sampling). 
#' 
#'   If individual counts were estimated by **sampling**, additional column(s) 
#'   providing a measure of precision is also required (e.g. `lower_ci` and 
#'   `upper_ci`, or `sd`, `cv`, `var`). Precision metrics can be different 
#'   between counts. For instance, some sampling counts can have a `sd` value 
#'   and others `lower_ci` and `upper_ci`. In that case three columns are 
#'   required (`lower_ci`, `upper_ci`, and `sd`). See above section 
#'   **Description** for further information on the computation of the 95% 
#'   confident interval of estimates. 
#' 
#'   If the individuals were counted by different methods, an additional field 
#'   `field_method` is also required. It can contain: `G` (ground counts) 
#'   and/or `A` (aerial counts). See above section **Description** for further 
#'   information on the counts conversion.
#'   
#'   Others fields can be present either in `data` or `info` (see below).
#' 
#' @param info (optional) a `data.frame` with species in rows and the following
#'   columns: `species` (species name), `pref_field_method`, 
#'   `conversion_A2G`, and `rmax`. See above section **Description** for 
#'   further information on these fields.
#'   Default is `NULL` (i.e. these information must be present in `data` 
#'   if not available in `popbayes`).
#' 
#' @param location a `character` string. The column name in `data` of the
#'   site. This field is used to distinguish count series from different sites
#'   (if required) and to create an unique series name.
#'   Default is `'location'`.
#'   
#' @param species a `character` string. The column name in `data` (and 
#'   in `info` if provided) of the species. This field is used to distinguish 
#'   count series for different species (if required) and to create an unique 
#'   series name.
#'   Default is `'species'`.
#'   
#' @param date a `character` string. The column name in `data` of the date.
#'   This column `date` must be in a numerical form with possibly a decimal 
#'   part.
#'   Default is `'date'`.
#'   
#' @param count a `character` string. The column name in `data` of the
#'   number of individuals. This column must be numerical.
#'   Default is `'count'`.
#'  
#' @param stat_method a `character` string. The column name in `data` of 
#'   the method used to estimate individuals counts. It can contain `'T'` 
#'   (total counts), `'X'` (guesstimate), and/or `'S'` (sampling). If some 
#'   counts are coded as `'S'`, precision column(s) must also be provided (see 
#'   below).
#'   Default is `'stat_method'`. 
#' 
#' @param lower_ci (optional) a `character` string. The column name in `data`
#'   of the lower boundary of the 95% CI of the estimate (i.e. `count`). If 
#'   provided, the upper boundary of the 95% CI (argument `upper_ci`) must be 
#'   also provided. This argument is only required if some counts have been 
#'   estimated by a sampling method. But user may prefer use other precision 
#'   measures, e.g. standard deviation (argument `sd`), variance (argument 
#'   `var`), or coefficient of variation (argument `cv`). 
#'   Default is `'lower_ci'`.
#'   
#' @param upper_ci (optional) a `character` string. The column name in `data`
#'   of the upper boundary of the 95% CI of the estimate (i.e. `count`). If 
#'   provided, the lower boundary of the 95% CI (argument `lower_ci`) must be 
#'   also provided.
#'   Default is `'upper_ci'`.
#'   
#' @param sd (optional) a `character` string. The column name in `data` of 
#'   the standard deviation of the estimate.
#'   Default is `NULL`.
#'   
#' @param var (optional) a `character` string. The column name in `data` of 
#'   the variance of the estimate.
#'   Default is `NULL`.
#'    
#' @param cv (optional) a `character` string. The column name in `data` of 
#'   the coefficient of variation of the estimate.
#'   Default is `NULL`.
#'   
#' @param field_method (optional) a `character` string. The column name in 
#'   `data` of the field method used to count individuals. Counts can be ground 
#'   counts (coded as `'G'`) or aerial counts (coded as `'A'`). This argument 
#'   is optional if individuals have been counted by the same method. See above 
#'   section **Description** for further information on the count conversion.
#'   Default is `NULL`.
#'   
#' @param pref_field_method (optional) a `character` string. The column name
#'   in `data` of the preferred field method of the species. This argument is
#'   only required is `field_method` is not `NULL` (i.e. individuals have been 
#'   counted by different methods). Alternatively, this value can be passed in
#'   `info` (or internally retrieved if the species is listed in the package). 
#'   See above section **Description** for further information on the count
#'   conversion.
#'   Default is `NULL`.
#' 
#' @param conversion_A2G (optional) a `character` string. The column name
#'   in `data` of the count conversion factor of the species. This argument is
#'   only required if `field_method` is not `NULL` (i.e. individuals have been 
#'   counted by different methods). Alternatively this value can be passed in
#'   `info` (or internally retrieved if the species is listed in the package).
#'   See above section **Description** for further information on the count
#'   conversion.
#'   Default is `NULL`.
#'   
#' @param rmax (optional) a `character` string. The column name in `data` of 
#'   the species demographic potential (i.e. the relative rate of increase of 
#'   the population). This is the change in log population size between two 
#'   dates and will be used later by [fit_trend()].
#'   Default is `NULL`.
#'   
#' @param path a `character` string. The directory to save formatted data. 
#'   This directory must exist and can be an absolute or a relative path.
#'   Default is the current working directory.
#' 
#' @param na_rm a `logical.` If `TRUE`, counts with `NA` values will be 
#'   removed.
#'   Default is `FALSE` (returns an error to inform user if `NA` are detected).
#'
#'
#'
#' @return An n-elements `list` (where `n` is the number of count series). The
#'   name of each element of this list is a combination of location and 
#'   species. Each element of the list is a `list` with the following content:
#'   \itemize{
#'   \item \code{location} a `character` string. The name of the series site.
#'   \item \code{species} a `character` string. The name of the series species.
#'   \item \code{date} a `numerical` vector. The sequence of dates of the 
#'     series.
#'   \item \code{n_dates} an `integer.` The number of unique dates.
#'   \item \code{stat_methods} a `character` vector. The different stat methods
#'     of the series.
#'   \item \code{field_methods} (optional) a `character` vector. The different
#'     field methods of the series.
#'   \item \code{pref_field_method} (optional) a `character` string. The 
#'     preferred field method of the species (`'A'` or `'G'`).
#'   \item \code{conversion_A2G} (optional) a `numeric`. The conversion factor
#'     of the species used to convert counts to its preferred field method.
#'   \item \code{rmax} a `numeric`. The maximum population growth rate of the
#'     species.
#'   \item \code{data_original} a `data.frame`. Original data of the series 
#'     with renamed columns. Some rows may have been deleted 
#'     (if `na_rm = TRUE`).
#'   \item \code{data_converted} a `data.frame`. Data containing computed 
#'     boundaries of the 95% CI (`lower_ci_conv` and `upper_ci_conv`). If 
#'     counts have been obtained by different field methods, contains also 
#'     converted counts (`count_conv`) based on the preferred field method and
#'     conversion factor of the species. This `data.frame` will be used by the
#'     function [fit_trend()] to fit population models.
#'   }
#'   
#'   **Note:** Some original series can be discarded if one of these two 
#'   conditions is met: 1) the series contains only zero counts, and 2) the 
#'   series contains only a few dates (< 4 dates).
#' 
#' @export
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#' 
#' ## Number of count series ----
#' length(garamba_formatted)
#' 
#' ## Retrieve count series names ----
#' popbayes::list_series(path = temp_path)
#' 
#' ## Print content of the first count series ----
#' names(garamba_formatted[[1]])
#' 
#' ## Print original data ----
#' garamba_formatted[[1]]$"data_original"
#' 
#' ## Print converted data ----
#' garamba_formatted[[1]]$"data_converted"

format_data <- function(data, info = NULL, date = "date", count = "count", 
                        location = "location", species = "species", 
                        stat_method = "stat_method", lower_ci = "lower_ci", 
                        upper_ci = "upper_ci", sd = NULL, var = NULL, 
                        cv = NULL, field_method = NULL, 
                        pref_field_method = NULL,
                        conversion_A2G = NULL, rmax = NULL, 
                        path = ".", na_rm = FALSE) {
  
  
  # CHECKS ----
  
  ## Check argument 'path' ----
  
  if (!is.character(path) || length(path) != 1) {
    stop("Argument 'path' must be a character string.")
  }
  
  if (!dir.exists(path)) {
    stop("The directory '", path, "' does not exist.")
  }
  
  
  ## Check argument 'data' ----
  
  if (missing(data)) {
    stop("Argument 'data' is required.")
  }
  
  if (!is.data.frame(data)) {
    stop("Argument 'data' must be a data.frame.")
  }
  
  
  ## Coerce to data.frame (tibble) ----
  
  data <- as.data.frame(data)
  
  
  ## Check argument 'location' ----
  
  if (!is.character(location) || length(location) != 1) {
    stop("Argument 'location' must be a column name (character string).")
  }
  
  if (!(location %in% colnames(data))) {
    stop("The column '", location, "' (argument location) is absent from ", 
         "'data'. Please check the spelling.")
  }
  
  data[ , location] <- as.character(data[ , location])
  
  if (any(is.na(data[ , location]))) {
    stop("The column '", location, "' cannot contain NA.")
  }
  
  
  ## Check argument 'species' ----
  
  if (!is.character(species) || length(species) != 1) {
    stop("Argument 'species' must be a column name (character string).")
  }
  
  if (!(species %in% colnames(data))) {
    stop("The column '", species, "' (argument species) is absent from ", 
         "'data'. Please check the spelling.")
  }
  
  data[ , species] <- as.character(data[ , species])
  
  if (any(is.na(data[ , species]))) {
    stop("The column '", species, "' cannot contain NA.")
  }
  
  
  ## Check argument 'date' ----
  
  if (!is.character(date) || length(date) != 1) {
    stop("Argument 'date' must be a column name (character string).")
  }
  
  if (!(date %in% colnames(data))) {
    stop("The column '", date, "' (argument date) is absent from 'data'. ", 
         "Please check the spelling.")
  }
  
  if (!is.numeric(data[ , date])) {
    stop("The column '", date, "' must be numeric.")
  }
  
  if (any(is.na(data[ , date]))) {
    stop("The column '", date, "' cannot contain NA.")
  }
  
  
  ## Check argument 'count' ----
  
  if (!is.character(count) || length(count) != 1) {
    stop("Argument 'count' must be a column name (character string).")
  }
  
  if (!(count %in% colnames(data))) {
    stop("The column '", count, "' (argument count) is absent from 'data'. ", 
         "Please check the spelling.")
  }
  
  if (!is.numeric(data[ , count])) {
    stop("The column '", count, "' must be numeric.")
  }
  
  if (length(which(data[ , count] < 0))) {
    stop("The column '", count, "' must be positive (or zero).")
  }
  
  
  ## Check argument 'na_rm' ----
  
  if (!is.logical(na_rm) || length(na_rm) != 1) {
    stop("Argument 'na_rm' must be TRUE or FALSE.")
  }
  
  
  ## Check argument 'stat_method' ----
  
  if (!is.character(stat_method) || length(stat_method) != 1) {
    stop("Argument 'stat_method' must be a column name (character string).")
  }
  
  if (!(stat_method %in% colnames(data))) {
    stop("The column '", stat_method, "' (argument stat_method) is absent ", 
         "from 'data'. Please check the spelling.")
  }
  
  valid_stat_methods <- c("T", "X", "S")
  
  valid_stat_methods_msg <- paste0(valid_stat_methods, collapse = "' or '")
  valid_stat_methods_msg <- paste0("'", valid_stat_methods_msg, "'")
  
  data[ , stat_method] <- as.character(data[ , stat_method])
  
  if (any(is.na(data[ , stat_method]))) {
    stop("The column '", stat_method, "' cannot contain NA.")
  }
  
  if (any(!(data[ , stat_method] %in% valid_stat_methods))) {
    stop("Invalid value(s) for 'stat_method' in 'data'. ",
         "Allowed values are: ", valid_stat_methods_msg, ".")
  }  
  
  
  ## Check argument 'lower_ci' ----
  
  if (!is.null(lower_ci)) {
    
    if (!is.character(lower_ci) || length(lower_ci) != 1) {
      stop("Argument 'lower_ci' must be a column name (character string).")
    }
    
    if (!(lower_ci %in% colnames(data))) {
      stop("The column '", lower_ci, "' (argument lower_ci) is absent ", 
           "from 'data'. Please check the spelling.")
    }
    
    if (!is.numeric(data[ , lower_ci])) {
      stop("The column '", lower_ci, "' must be numeric.")
    }
    
    if (length(which(data[ , lower_ci] < 0))) {
      stop("The column '", lower_ci, "' must be positive (or zero).")
    }
  }
  
  
  ## Check argument 'upper_ci' ----
  
  if (!is.null(upper_ci)) {
    
    if (!is.character(upper_ci) || length(upper_ci) != 1) {
      stop("Argument 'upper_ci' must be a column name (character string).")
    }
    
    if (!(upper_ci %in% colnames(data))) {
      stop("The column '", upper_ci, "' (argument upper_ci) is absent ", 
           "from 'data'. Please check the spelling.")
    }
    
    if (!is.numeric(data[ , upper_ci])) {
      stop("The column '", upper_ci, "' must be numeric.")
    }
    
    if (length(which(data[ , upper_ci] < 0))) {
      stop("The column '", upper_ci, "' must be positive (or zero).")
    }
  }
  
  
  ## Check argument 'sd' ----
  
  if (!is.null(sd)) {
    
    if (!is.character(sd) || length(sd) != 1) {
      stop("Argument 'sd' must be a column name (character string).")
    }
    
    if (!(sd %in% colnames(data))) {
      stop("The column '", sd, "' (argument sd) is absent from 'data'. ",
           "Please check the spelling.")
    }
    
    if (!is.numeric(data[ , sd])) {
      stop("The column '", sd, "' must be numeric.")
    }
    
    if (length(which(data[ , sd] <= 0))) {
      stop("The column '", sd, "' must be strictly positive.")
    }
  }
  
  
  ## Check argument 'var' ----
  
  if (!is.null(var)) {
    
    if (!is.character(var) || length(var) != 1) {
      stop("Argument 'var' must be a column name (character string).")
    }
    
    if (!(var %in% colnames(data))) {
      stop("The column '", var, "' (argument var) is absent from 'data'. ",
           "Please check the spelling.")
    }
    
    if (!is.numeric(data[ , var])) {
      stop("The column '", var, "' must be numeric.")
    }
    
    if (length(which(data[ , var] <= 0))) {
      stop("The column '", var, "' must be strictly positive.")
    }
  } 
  
  
  ## Check argument 'cv' ----
  
  if (!is.null(cv)) {
    
    if (!is.character(cv) || length(cv) != 1) {
      stop("Argument 'cv' must be a column name (character string).")
    }
    
    if (!(cv %in% colnames(data))) {
      stop("The column '", cv, "' (argument cv) is absent from 'data'. ",
           "Please check the spelling.")
    }
    
    if (!is.numeric(data[ , cv])) {
      stop("The column '", cv, "' must be numeric.")
    }
    
    if (length(which(data[ , cv] <= 0))) {
      stop("The column '", cv, "' must be strictly positive.")
    }
  } 
  
  
  ## Check argument 'field_method' ----
  
  if (!is.null(field_method)) {
    
    if (!is.character(field_method) || length(field_method) != 1) {
      stop("Argument 'field_method' must be a column name (character of ",
           "length 1).")
    }
    
    if (!(field_method %in% colnames(data))) {
      stop("The column '", field_method, "' (argument field_method) is ", 
           "absent from 'data'. Please check the spelling.")
    }
    
    valid_field_methods <- c("G", "A")
    
    valid_field_methods_msg <- paste0(valid_field_methods, collapse = "' or '")
    valid_field_methods_msg <- paste0("'", valid_field_methods_msg, "'")
    
    data[ , field_method] <- as.character(data[ , field_method])
    
    data_not_x <- data[data[ , stat_method] != "X", ]
    
    if (nrow(data_not_x)) {
      
      if (any(is.na(data_not_x[ , field_method]))) {
        stop("The column '", field_method, "' cannot contain NA ", 
             "(except for count with stat_method = 'X').")
      } 
      
      if (any(!(data_not_x[ , field_method] %in% valid_field_methods))) {
        stop("Invalid value(s) for 'field_method' in 'data'. ",
             "Allowed values are: ", valid_field_methods_msg, ".")
      }
    }
  }
  
  
  ## Check argument 'pref_field_method' ----
  
  if (!is.null(pref_field_method)) {
    
    if (!is.character(pref_field_method) || length(pref_field_method) != 1) {
      stop("Argument 'pref_field_method' must be a column name (character of ",
           "length 1).")
    }
    
    if (!(pref_field_method %in% colnames(data))) {
      stop("The column '", pref_field_method, "' (argument pref_field_method) ",
           "is absent from 'data'. Please check the spelling.")
    }
    
    valid_field_methods <- c("G", "A")
    
    valid_field_methods_msg <- paste0(valid_field_methods, collapse = "' or '")
    valid_field_methods_msg <- paste0("'", valid_field_methods_msg, "'")
    
    data[ , pref_field_method] <- as.character(data[ , pref_field_method])
    
    data_not_na <- data[!is.na(data[ , pref_field_method]), ]
    
    if (nrow(data_not_na)) {
      
      if (any(!(data_not_na[ , pref_field_method] %in% valid_field_methods))) {
        stop("Invalid value(s) for 'field_method' in 'data'. ",
             "Allowed values are: ", valid_field_methods_msg, ".")
      } 
    }
  }
  
  
  ## Check argument 'conversion_A2G' ----
  
  if (!is.null(conversion_A2G)) {
    
    if (!is.character(conversion_A2G) || length(conversion_A2G) != 1) {
      stop("Argument 'conversion_A2G' must be a column name (character of ",
           "length 1).")
    }
    
    if (!(conversion_A2G %in% colnames(data))) {
      stop("The column '", conversion_A2G, "' (argument conversion_A2G) ", 
           "is absent from 'data'. Please check the spelling.")
    }
    
    if (!is.numeric(data[ , conversion_A2G])) {
      stop("The column '", conversion_A2G, "' must be a numeric.")
    }
    
    if (length(which(data[ , conversion_A2G] <= 0))) {
      stop("The column '", conversion_A2G, "' must be strictly positive.")
    }
  }
  
  
  ## Check argument 'rmax' ----
  
  if (!is.null(rmax)) {
    
    if (!is.character(rmax) || length(rmax) != 1) {
      stop("Argument 'rmax' must be a column name (character of ",
           "length 1).")
    }
    
    if (!(rmax %in% colnames(data))) {
      stop("The column '", rmax, "' (argument rmax) ", 
           "is absent from 'data'. Please check the spelling.")
    }
    
    if (!is.numeric(data[ , rmax])) {
      stop("The column '", rmax, "' must be a numeric.")
    }
    
    if (length(which(data[ , rmax] <= 0))) {
      stop("The column '", rmax, "' must be strictly positive.")
    }
  }
  
  
  ## Check argument 'info' ----
  
  if (!is.null(info)) {
    
    if (!is.data.frame(info)) {
      stop("Argument 'info' must be a data frame.")
    }
    
    valid_info_colnames <- c("species", "pref_field_method", "conversion_A2G",
                             "rmax")
    
    valid_info_colnames_msg <- paste0(valid_info_colnames, collapse = "' and '")
    valid_info_colnames_msg <- paste0("'", valid_info_colnames_msg, "'")
    
    
    if (any(!(valid_info_colnames %in% colnames(info)))) {
      stop("Invalid columns in 'info'. ",
           "Required variables are: ", valid_info_colnames_msg, ".")
    }
    
    if (any(is.na(info[ , valid_info_colnames]))) {
      stop("The dataset 'info' cannot contain NA.")
    }
    
    if (length(which(duplicated(info$"species"))) > 0) {
      stop("The 'info' data.frame cannot contain duplicated species.")
    }
  }
  
  
  ## Check precision information ----
  
  if ("S" %in% data[ , stat_method]) {
    
    if (is.null(lower_ci) && is.null(upper_ci) && is.null(sd) && is.null(cv) &&
        is.null(var)) {
      
      stop("No valid measure of precision is available for sampling counts. ", 
           "Add 'lower_ci' and 'upper_ci' and/or 'sd', 'var', 'cv' ", 
           "information.")
    }
    
    if (!is.null(lower_ci) && is.null(upper_ci)) {
      stop("You must provide both lower and upper CI.")
    }
    
    if (is.null(lower_ci) && !is.null(upper_ci)) {
      stop("You must provide both lower and upper CI.")
    }
  }
  
 
  # NA COUNTS ----
  
  ## Handle NA in counts ----
  
  data <- is_na_counts(data, count, na_rm)
  
  if (nrow(data) == 0) {
    stop("All counts are NA. Please check your data.")
  }
  
  
  # FORMAT DATA ----
  
  ## Detect precision(s) columns ----
  
  precision_cols <- NULL
  
  if ("S" %in% data[ , stat_method]) {
    
    if (!is.null(lower_ci)) {
      
      precision_cols <- c(precision_cols, lower_ci)
      names(precision_cols)[length(precision_cols)] <- "lower_ci"
    }
    
    if (!is.null(upper_ci)) {
      
      precision_cols <- c(precision_cols, upper_ci)
      names(precision_cols)[length(precision_cols)] <- "upper_ci"
    }
    
    if (!is.null(sd)) {
      
      precision_cols <- c(precision_cols, sd)
      names(precision_cols)[length(precision_cols)] <- "sd"
    }
    
    if (!is.null(var)) {
      
      precision_cols <- c(precision_cols, var)
      names(precision_cols)[length(precision_cols)] <- "var"
    }
    
    if (!is.null(cv)) {
      
      precision_cols <- c(precision_cols, cv)
      names(precision_cols)[length(precision_cols)] <- "cv"
    }
  }
  
  
  ## Rename columns ----
  
  data_renamed <- data[ , c(location, species, date, stat_method)]
  colnames(data_renamed) <- c("location", "species", "date", "stat_method")
  
  if (!is.null(field_method)) {
    data_renamed <- data.frame(data_renamed, 
                               "field_method" = data[ , field_method])
  }
  
  if (!is.null(pref_field_method)) {
    data_renamed <- data.frame(data_renamed, 
                               "pref_field_method" = data[ , pref_field_method])
  }
  
  if (!is.null(conversion_A2G)) {
    data_renamed <- data.frame(data_renamed, 
                               "conversion_A2G" = data[ , conversion_A2G])
  }
  
  if (!is.null(rmax)) {
    data_renamed <- data.frame(data_renamed, 
                               "rmax" = data[ , rmax])
  }
  
  data_renamed <- data.frame(data_renamed, 
                             "count_orig" = data[ , count])
  
  if (!is.null(precision_cols)) {
    
    precision_data <- data[ , precision_cols]
    precision_cols <- paste0(names(precision_cols), "_orig")
    colnames(precision_data) <- precision_cols
    
    data_renamed <- data.frame(data_renamed, precision_data)
  }
  
  
  # CREATE SERIES ----
  
  ## Detect series ----
  
  series_infos <- get_series(data_renamed, quiet = TRUE)
  
  
  ## Split data by series ----
  
  count_series <- list()
  
  for (i in seq_len(nrow(series_infos))) {
    
    id <- series_infos[i, "id"]
    
    sel_rows <- which(data_renamed$"location" == series_infos[i, "location"] &
                        data_renamed$"species"  == series_infos[i, "species"])
    
    series <- data_renamed[sel_rows, ]
    rownames(series) <- NULL
    
    count_series[[id]] <- series
  }
  
  
  # SPECIES INFO ----
  
  for (i in seq_along(count_series)) {
    
    if (!is.null(field_method)) {
      
      
      ## Retrieve 'pref_field_method' ----
      
      if (!is.null(pref_field_method)) {
        
        pref_field_method_data <- unique(count_series[[i]]$"pref_field_method")
        pref_field_method_data <- pref_field_method_data[
          !is.na(pref_field_method_data)]
        
        if (length(pref_field_method_data) > 1) {
          stop("Multiple values for 'pref_field_method' detected in ", 
               names(count_series)[i], ".")
        }

        if (length(pref_field_method_data) == 0) {
          
          if (!is.null(info)) {
            
            if (unique(count_series[[i]]$"species") %in% info$"species") {
              
              pos <- info$"species" %in% count_series[[i]]$"species"[1]
              pref_field_method_data <- info[pos, "pref_field_method"]
              
            } else {
              
              if (unique(count_series[[i]]$"species") %in% 
                  species_info$"species") {
                
                pos <- species_info$"species" %in% 
                  count_series[[i]]$"species"[1]
                pref_field_method_data <- species_info[pos, "pref_field_method"]
                
              } else {
                
                stop("Unable to retrieve 'pref_field_method' for the series ", 
                     names(count_series)[i])
              }
            }
            
          } else {
            
            if (unique(count_series[[i]]$"species") %in% 
                species_info$"species") {
              
              pos <- species_info$"species" %in% count_series[[i]]$"species"[1]
              pref_field_method_data <- species_info[pos, "pref_field_method"]
              
            } else {
              
              stop("Unable to retrieve 'pref_field_method' for the series ", 
                   names(count_series)[i])
            }
          }
        }
        
      } else {
        
        if (!is.null(info)) {
          
          if (unique(count_series[[i]]$"species") %in% info$"species") {
            
            pos <- info$"species" %in% count_series[[i]]$"species"[1]
            pref_field_method_data <- info[pos, "pref_field_method"]
            
          } else {
            
            if (unique(count_series[[i]]$"species") %in% 
                species_info$"species") {
              
              pos <- species_info$"species" %in% count_series[[i]]$"species"[1]
              pref_field_method_data <- species_info[pos, "pref_field_method"]
              
            } else {
              
              stop("Unable to retrieve 'pref_field_method' for the series ", 
                   names(count_series)[i])
            }
          }
          
        } else {
          
          if (unique(count_series[[i]]$"species") %in% 
              species_info$"species") {
            
            pos <- species_info$"species" %in% count_series[[i]]$"species"[1]
            pref_field_method_data <- species_info[pos, "pref_field_method"]
            
          } else {
            
            stop("Unable to retrieve 'pref_field_method' for the series ", 
                 names(count_series)[i])
          }
        }
      }
      
      count_series[[i]]$"pref_field_method" <- pref_field_method_data
      
      
      ## Retrieve 'conversion_A2G' ----
      
      if (!is.null(conversion_A2G)) {
      
        conversion_A2G_data <- unique(count_series[[i]]$"conversion_A2G")
        conversion_A2G_data <- conversion_A2G_data[
          !is.na(conversion_A2G_data)]
        
        if (length(conversion_A2G_data) > 1) {
          stop("Multiple values for 'conversion_A2G' detected in ", 
               names(count_series)[i], ".")
        }

        if (length(conversion_A2G_data) == 0) {
          
          if (!is.null(info)) {
            
            if (unique(count_series[[i]]$"species") %in% info$"species") {
              
              pos <- info$"species" %in% count_series[[i]]$"species"[1]
              conversion_A2G_data <- info[pos, "conversion_A2G"]
              
            } else {
              
              if (unique(count_series[[i]]$"species") %in% 
                  species_info$"species") {
                
                pos <- species_info$"species" %in% 
                  count_series[[i]]$"species"[1]
                conversion_A2G_data <- species_info[pos, "conversion_A2G"]
                
              } else {
                
                stop("Unable to retrieve 'conversion_A2G' for the series ", 
                     names(count_series)[i])
              }
            }
            
          } else {
            
            if (unique(count_series[[i]]$"species") %in% 
                species_info$"species") {
              
              pos <- species_info$"species" %in% count_series[[i]]$"species"[1]
              conversion_A2G_data <- species_info[pos, "conversion_A2G"]
              
            } else {
              
              stop("Unable to retrieve 'conversion_A2G' for the series ", 
                   names(count_series)[i])
            }
          }
        }
        
      } else {
        
        if (!is.null(info)) {
          
          if (unique(count_series[[i]]$"species") %in% info$"species") {
            
            pos <- info$"species" %in% count_series[[i]]$"species"[1]
            conversion_A2G_data <- info[pos, "conversion_A2G"]
            
          } else {
            
            if (unique(count_series[[i]]$"species") %in% 
                species_info$"species") {
              
              pos <- species_info$"species" %in% count_series[[i]]$"species"[1]
              conversion_A2G_data <- species_info[pos, "conversion_A2G"]
              
            } else {
              
              stop("Unable to retrieve 'conversion_A2G' for the series ", 
                   names(count_series)[i])
            }
          }
          
        } else {
          
          if (unique(count_series[[i]]$"species") %in% 
              species_info$"species") {
            
            pos <- species_info$"species" %in% count_series[[i]]$"species"[1]
            conversion_A2G_data <- species_info[pos, "conversion_A2G"]
            
          } else {
            
            stop("Unable to retrieve 'conversion_A2G' for the series ", 
                 names(count_series)[i])
          }
        }
      }
      
      count_series[[i]]$"conversion_A2G"    <- conversion_A2G_data
    }
    
    
    ## Retrieve 'rmax' ----
    
    if (!is.null(rmax)) {
      
      rmax_data <- unique(count_series[[i]]$"rmax")
      rmax_data <- rmax_data[!is.na(rmax_data)]
      
      if (length(rmax_data) > 1) {
        stop("Multiple values for 'rmax' detected in ", 
             names(count_series)[i], ".")
      }
      
      
      if (length(rmax_data) == 0) {
        
        if (!is.null(info)) {
          
          if (unique(count_series[[i]]$"species") %in% info$"species") {
            
            pos <- info$"species" %in% count_series[[i]]$"species"[1]
            rmax_data <- info[pos, "rmax"]
            
          } else {
            
            if (unique(count_series[[i]]$"species") %in% 
                species_info$"species") {
              
              pos <- species_info$"species" %in% count_series[[i]]$"species"[1]
              rmax_data <- species_info[pos, "rmax"]
              
            } else {
              
              stop("Unable to retrieve 'rmax' for the series ", 
                   names(count_series)[i])
            }
          }
          
        } else {
          
          if (unique(count_series[[i]]$"species") %in% 
              species_info$"species") {
            
            pos <- species_info$"species" %in% count_series[[i]]$"species"[1]
            rmax_data <- species_info[pos, "rmax"]
            
          } else {
            
            stop("Unable to retrieve 'rmax' for the series ", 
                 names(count_series)[i])
          }
        }
      }
      
    } else {
      
      if (!is.null(info)) {
        
        if (unique(count_series[[i]]$"species") %in% info$"species") {
          
          pos <- info$"species" %in% count_series[[i]]$"species"[1]
          rmax_data <- info[pos, "rmax"]
          
        } else {
          
          if (unique(count_series[[i]]$"species") %in% 
              species_info$"species") {
            
            pos <- species_info$"species" %in% count_series[[i]]$"species"[1]
            rmax_data <- species_info[pos, "rmax"]
            
          } else {
            
            stop("Unable to retrieve 'rmax' for the series ", 
                 names(count_series)[i])
          }
        }
        
      } else {
        
        if (unique(count_series[[i]]$"species") %in% 
            species_info$"species") {
          
          pos <- species_info$"species" %in% count_series[[i]]$"species"[1]
          rmax_data <- species_info[pos, "rmax"]
          
        } else {
          
          stop("Unable to retrieve 'rmax' for the series ", 
               names(count_series)[i])
        }
      }
    }
    
    count_series[[i]]$"rmax" <- rmax_data
  }
  
  
  # CHECK PRECISION MEASURES ----
  
  for (i in seq_along(count_series)) {
    
    count_series[[i]] <- is_na_precision(count_series[[i]], precision_cols, 
                                         na_rm)
  }
  
  
  ## Remove series ----
  
  n_rows <- unlist(lapply(count_series, function(x) nrow(x)))
  n_rows <- n_rows[n_rows == 0]
  
  if (length(n_rows)) {
    
    series_to_del <- names(n_rows)
    count_series <- count_series[-which(names(count_series) %in% series_to_del)]
    
    if (length(n_rows) < length(count_series)) {
      cli::cli_alert_danger(c(
        "Removing the following series without valid precision measures: ",
        "{.val {series_to_del}}"
      ))
    }
  }
  
  
  n_rows <- unlist(lapply(count_series, function(x) nrow(x)))
  n_rows <- n_rows[n_rows < 4]
  
  if (length(n_rows)) {
    
    series_to_del <- names(n_rows)
    
    if (!na_rm) {
      
      cli::cli_abort(c(
        "The following count series have not enough data (< 4):", 
        "{.val {series_to_del}}",
        i = "Remove these count series or use 'na_rm = TRUE'."
      ))
      
    } else {
      
      count_series <- count_series[-which(names(count_series) %in% 
                                            series_to_del)]  
    }
    
    if (length(n_rows) < length(count_series)) {
      cli::cli_alert_danger(c(
        "Removing the following series without enough data (< 4) ",
        "{.val {series_to_del}}"
      ))
    }
  }
  
  if (length(count_series) == 0) {
    stop("All your series have been removed. Please check your data.")
  }
  
  
  # COMPUTE CI BOUNDARIES ----
  
  for (i in seq_along(count_series)) {
    
    count_series[[i]] <- compute_ci(count_series[[i]], precision_cols)
  }
  
  
  # CONVERT COUNTS ----
  
  for (i in seq_along(count_series)) {
  
    count_series[[i]] <- convert_counts(count_series[[i]], field_method)
  }
  
  
  # ZERO COUNTS ----
  
  for (i in seq_along(count_series)) {
    
    count_series[[i]] <- zero_counts(count_series[[i]], na_rm)
  }
  
  n_rows <- unlist(lapply(count_series, function(x) nrow(x)))
  n_rows <- n_rows[n_rows == 0]
  
  if (length(n_rows)) {
    
    series_to_del <- names(n_rows)
    count_series <- count_series[-which(names(count_series) %in% series_to_del)]
    
    if (length(n_rows) < length(count_series)) {
      cli::cli_alert_danger(c(
        "Removing the following series without valid precision measures: ",
        "{.val {series_to_del}}"
      ))
    }
  }
  
  if (length(count_series) == 0) {
    stop("All your series have been removed. Please check your data.")
  }
  
  
  # FINAL CHECKS ----
  
  for (i in seq_along(count_series)) {
    
    if (any(count_series[[i]][ , "lower_ci_conv"] ==
            count_series[[i]][ , "upper_ci_conv"])) {
      cli::cli_abort(c(
        "Lower and upper CI bounds cannot be strictly equal for the series.",
        "{.val {names(count_series)[i]}}"
      ))
    }
  }
  
  
  # CREATE FINAL LIST ----
  
  data_series    <- list()
  
  for (i in seq_len(nrow(series_infos))) {
    
    id <- names(count_series)[i]
    
    data_sub <- count_series[[i]][order(count_series[[i]]$"date",
                                        decreasing = FALSE), ]
    rownames(data_sub) <- NULL
    
    if (!is.null(field_method)) {
      
      pref_field_method <- unique(data_sub$"pref_field_method")
      conversion_A2G    <- unique(data_sub$"conversion_A2G")
      
      field_methods <- sort(unique(data_sub[ , "field_method"]))
      field_methods <- field_methods[!is.na(field_methods)]
      
    } else {
      
      pref_field_method <- NULL
      conversion_A2G    <- NULL
      field_methods     <- NULL
    }
      
    rmax <- unique(data_sub$"rmax")
      
      
    data_series[[id]] <- list(
      "location"          = unique(data_sub$"location"),
      "species"           = unique(data_sub$"species"),
      "dates"             = data_sub[ , "date"],
      "n_dates"           = length(unique(data_sub[ , "date"])),
      "stat_methods"      = sort(unique(data_sub[ , "stat_method"])),
      "field_methods"     = field_methods,
      "pref_field_method" = pref_field_method,
      "conversion_A2G"    = conversion_A2G,
      "rmax"              = rmax,
      "data_original"     = data_sub[ , -grep("_conv", colnames(data_sub))],
      "data_converted"    = data_sub[ , -grep("_orig", colnames(data_sub))])
    
    
    ## Export sub-list ----
    
    species_path <- file.path(path, id)
    dir.create(species_path, showWarnings = FALSE)
    
    formatted_data <- data_series[id]
    save(formatted_data, file = file.path(species_path,
                                          paste0(id, "_data.RData")))
  }
  
  
  cli::cli_alert_success(
    "Detecting {.val {length(data_series)}} count series."
  )
  
  data_series
}



#' Remove rows or return error if NA counts detected
#'
#' @param data a `data.frame`
#' @param col a `character` string (column to inspect)
#' @param na_rm a `logical`. If `TRUE`, deletes rows. Otherwise, returns an 
#'   error.
#'
#' @return A `data.frame` (same as `data` with possibly some rows removed).
#' 
#' @noRd

is_na_counts <- function(data, col, na_rm) {
  
  if (any(is.na(data[ , col]))) {
    
    if (!na_rm) {
      
      stop("The column '", col, "' cannot contain NA. If you want to ", 
           "remove missing counts, please use 'na_rm = TRUE'.", call. = FALSE)
      
    } else {
      
      pos <- which(is.na(data[ , col]))
      
      cli::cli_alert_info(c(
        "Removing {.val {length(pos)}} rows with NA values in {.field {col}} field."
      ))
      
      data <- data[-pos, ]
    }
  }
  
  data
}



#' Check precision data for sampling counts
#'
#' - Check for missing precision values: if neither ci, sd, var, cv are found, 
#'   returns an error (`na_rm = FALSE`) or deletes rows (`na_rm = TRUE`).
#' - Check for CI boundaries: lower and upper boundaries are required. If not, 
#'   returns an error.
#' - Check CI boundaries values: if lower_ci > count or upper_ci < count,
#'   returns an error.
#'
#' @param data a `data.frame`
#' @param precision_cols a `character` vector (column names of precision 
#'   information, i.e. `'lower_ci_orig'`, `'sd_orig'`, etc.)
#' @param na_rm a `logical`. If `TRUE` delete rows. Otherwise return an error.
#'
#' @return A data frame (same as `data`).
#' 
#' @noRd

is_na_precision <- function(data, precision_cols, na_rm) {
  
  
  ## Check for missing precision values ----
  
  sampling_rows <- which(data[ , "stat_method"] == "S")
  
  if (length(sampling_rows)) {
    
    is_na <- apply(data[sampling_rows, precision_cols], 1, function(x) {
      x <- sum(ifelse(is.na(x), 0, 1))
      ifelse(x == 0, TRUE, FALSE)
    })
  
  
    ## Remove or stop if missing precision values for S ----
    
    if (sum(is_na)) {
      
      if (!na_rm) {
        
        stop("Precision column(s) cannot all be NA for sampling counts. If ", 
             "you want to remove counts missing precision information, please ",
             "use 'na_rm = TRUE'.")
        
      } else {
        
        data <- data[-sampling_rows[which(is_na)], ]
      }
    }
  }
  
  if (nrow(data)) {
    
    
    ## Check for CI bounds (require both) for S ----
    
    sampling_rows <- which(data[ , "stat_method"] == "S")
    
    if (length(sampling_rows)) {
      
      if ("lower_ci_orig" %in% colnames(data)) {
        
        is_na_lower <- is.na(data[sampling_rows, "lower_ci_orig"])
        is_na_upper <- is.na(data[sampling_rows, "upper_ci_orig"])
        
        pos <- which((is_na_upper + is_na_lower) == 1)
        
        if (length(pos)) {
          
          ci_cols <- which(precision_cols %in% c("lower_ci_orig", 
                                                 "upper_ci_orig"))
          
          tmp <- as.data.frame(data[sampling_rows[pos], 
                                    precision_cols[-ci_cols]])
          
          if (ncol(tmp)) {
            
            is_na <- apply(tmp, 1, function(x) {
              x <- sum(ifelse(is.na(x), 0, 1))
              ifelse(x == 0, TRUE, FALSE)
            })
            
            if (sum(is_na)) {
              stop("Unless another type of precision information is provided, ",
                   "both lower and upper CI bounds are required.")
            }
            
          } else {
            
            stop("Unless another type of precision information is provided, ", 
                 "both lower and upper CI bounds are required.")
          }
        }
      }
    }
   
    
    ## Check CI bounds values for S ----
    
    if (length(sampling_rows)) {
      
      if ("lower_ci_orig" %in% colnames(data)) {
        
        pos <- which(data[sampling_rows, "lower_ci_orig"] > 
                       data[sampling_rows, "count_orig"])
        
        if (length(pos)) {
          stop("At least one CI lower bound is greater than the corresponding ",
               "count.")
        }
        
        
        pos <- which(data[sampling_rows, "lower_ci_orig"] < 0)
        
        if (length(pos)) {
          stop("CI lower bounds must be positive.")
        }
      }
      
      
      if ("upper_ci_orig" %in% colnames(data)) {
        
        pos <- which(data[sampling_rows, "upper_ci_orig"] < 
                       data[sampling_rows, "count_orig"])
        
        if (length(pos)) {
          stop("At least one CI upper bound is smaller than the corresponding ",
               "count.")
        }
        
        
        pos <- which(data[sampling_rows, "upper_ci_orig"] < 0)
        
        if (length(pos)) {
          stop("Upper CI values must be positive.")
        }
      }
    }
  }
  
  data
}



#' Compute 95% confident interval boundaries
#'
#' - Always computes CI bounds for stat methods T and G
#' - Derives CI bounds for stat method S from SD, VAR, or CV, unless CI bounds 
#'   are provided
#'
#' @param data a `data.frame`
#' @param precision_cols a `character` vector (names of columns with precision 
#'   information, i.e. `'lower_ci_orig'`, `'sd_orig'`, etc.)
#'
#' @return A `data.frame` identical to `data` with two additional columns: 
#'   `lower_ci_conv` and `upper_ci_conv`.
#' 
#' @noRd

compute_ci <- function(data, precision_cols) {
  
  data$"count_conv"    <- data$"count_orig"
  data$"lower_ci_conv" <- NA
  data$"upper_ci_conv" <- NA
  
  
  ## Compute CI boundaries for Total counts ----
  
  pos <- which(data$"stat_method" == "T")
  
  if (length(pos)) {
    data[pos, "lower_ci_conv"] <- data[pos, "count_orig"] * 0.95
    data[pos, "upper_ci_conv"] <- data[pos, "count_orig"] * 1.20
  }
  
  
  ## Compute CI boundaries for Guesstimates ----
  
  pos <- which(data$"stat_method" == "X")
  
  if (length(pos)) {
    data[pos, "lower_ci_conv"] <- data[pos, "count_orig"] * 0.80
    data[pos, "upper_ci_conv"] <- data[pos, "count_orig"] * 1.20
  }
  
  
  ## Compute CI boundaries for Sampling counts ----
  
  pos <- which(data$"stat_method" == "S")
  
  if (length(pos)) {
    
    for (i in pos) {
      
      found <- 0
      
      if (found == 0) {
        if ("lower_ci_orig" %in% precision_cols) {
          if (!is.na(data[i, "lower_ci_orig"])) {          # CI bounds
            data[i, "lower_ci_conv"] <- data[i, "lower_ci_orig"]
            data[i, "upper_ci_conv"] <- data[i, "upper_ci_orig"]
            found <- 1
          }
        }
      }
      
      if (found == 0) {
        if ("sd_orig" %in% precision_cols) {
          if (!is.na(data[i, "sd_orig"])) {                # Standard deviation
            data[i, "lower_ci_conv"] <- 
              data[i, "count_orig"] - 1.96 * data[i, "sd_orig"]
            data[i, "upper_ci_conv"] <- 
              data[i, "count_orig"] + 1.96 * data[i, "sd_orig"]
            found <- 1
          }
        }
      }
      
      if (found == 0) {
        if ("var_orig" %in% precision_cols) {
          if (!is.na(data[i, "var_orig"])) {               # Variance
            data[i, "lower_ci_conv"] <- 
              data[i, "count_orig"] - 1.96 * sqrt(data[i, "var_orig"])
            data[i, "upper_ci_conv"] <- 
              data[i, "count_orig"] + 1.96 * sqrt(data[i, "var_orig"])
            found <- 1
          }
        }
      }
      
      if (found == 0) {
        if ("cv_orig" %in% precision_cols) {
          if (!is.na(data[i, "cv_orig"])) {                # Coeff of variation
            data[i, "lower_ci_conv"] <- 
              data[i, "count_orig"] * (1 - 1.96 * data[i, "cv_orig"])
            data[i, "upper_ci_conv"] <- 
              data[i, "count_orig"] * (1 + 1.96 * data[i, "cv_orig"])
            found <- 1
          }
        }
      }
    }
  }
  
  data
}



#' Convert counts
#'
#' This function converts counts (and 95% CI lower and upper bounds) based on
#' the preferred field method and the species conversion factor.
#' 
#' **Important:** if the preferred field method is provided and the 
#' `field_method` column is present in `data`, counts are always converted 
#' toward the preferred field method.
#'
#' @param data a `data.frame`. Counts dataset.
#' 
#' @param field_method a `character` string. The column name in `data`.
#'
#' @return A `data.frame` (same as `data`).
#' 
#' @noRd

convert_counts <- function(data, field_method) {
  
  if (!is.null(field_method)) {
    
    method_pref <- unique(data$"pref_field_method")
    conv_fact   <- unique(data$"conversion_A2G")
    
    methods_used <- data$"field_method"
    
    # for guesstimates (only)
    methods_used <- ifelse(is.na(methods_used), method_pref, methods_used)
    
    conv_fact   <- ifelse(method_pref == "A", 1 / conv_fact, conv_fact)
    conv_fact   <- ifelse(methods_used == method_pref, 1, conv_fact)
    
    data[ , "count_conv"]   <- 
      data[ , "count_conv"]    * conv_fact
    data[ , "lower_ci_conv"] <- 
      data[ , "lower_ci_conv"] * conv_fact
    data[ , "upper_ci_conv"] <- 
      data[ , "upper_ci_conv"] * conv_fact
    
    data[ , "field_method_conv"] <- method_pref
  }
  
  data
}



#' Special cases: zero counts
#'
#' For a count series, identify zero counts.
#' 
#' If there are only zero counts, returns an error (`na_rm = FALSE`) or delete 
#' series (`na_rm = TRUE`). 
#' 
#' If there are zero and non-zero counts, replaces 0 counts by the smaller 
#' non-zero count (and replaces `lower_ci_conv` and `upper_ci_conv` by 
#' the corresponding values).
#'
#' @param data a `data.frame`
#' 
#' @param na_rm a `logical`. If `TRUE` delete series with all 0 counts. 
#'   Otherwise, return an error.
#'
#' @return A `data.frame` (same as `data`).
#' 
#' @noRd

zero_counts <- function(data, na_rm) {
  
  pos <- which(data[ , "count_conv"] == 0)
  
  if (length(pos)) {
    
    if (length(pos) == nrow(data)) {
      
      if (!na_rm) {
        
        stop("Some series have only zero counts. Check your data or ", 
             "'use na_rm = TRUE'.")
        
      } else {
        
        return(data[-pos, ])
      }
      
    } else {
      
      non_zero_counts  <- data[-pos, ]
      which_min_counts <- which.min(non_zero_counts[ , "count_conv"])[1]
      
      data[pos, "count_conv"]    <- non_zero_counts[which_min_counts, 
                                                    "count_conv"]
      data[pos, "lower_ci_conv"] <- non_zero_counts[which_min_counts, 
                                                    "lower_ci_conv"]
      data[pos, "upper_ci_conv"] <- non_zero_counts[which_min_counts, 
                                                    "upper_ci_conv"]
    }
  }
  
  data
}

#' Generate initial values of relative rate of increase (r)
#'
#' @description
#' This function extracts the counts and the corresponding dates from the list 
#' of objects (`data`) to be analyzed by the Bayesian model. It calculates the 
#' observed annual relative rate of increase (_r\_cand\_obs_) for each > 0
#' interval as the ratio of the change in log count divided by the interval 
#' length. It then draws a random initial value of the candidate relative rate 
#' of increase (_rcand_) to be passed to the bugs model as initial value by 
#' drawing in a normal distribution centered on the observed rate  
#' (_r\_cand\_obs_) with a standard deviation of 1. For each _r\_cand\_obs_, 
#' `nc` random values are drawn where `nc` (second argument) is the number of 
#' Markov chains.
#'
#' @param data a `list` created by [fit_trend()]
#' 
#' @param nc a positive `integer`. The number of MCMC chains.
#'
#' @return A `list` of `nc` lists (one per chain). Each list has exactly one 
#' element `rcand`, i.e. a vector of random initial values of candidate 
#' relative rates of increase, one per > 0 interval.    
#'
#' @noRd

inits_plus <- function(data, nc) {
  
  if (!is.list(data)) {
    stop("Argument 'data' must be an output of format_data().")
  }
  
  if (missing(nc)) {
    stop("Argument 'nc' is required.")
  }
  
  if (!is.numeric(nc) || length(nc) != 1) {
    stop("Argument 'nc' must be an integer of length 1.")
  }
  
  if (nc < 1) {
    stop("Argument 'nc' must be a positive integer.")
  }
  
  
  if (!("c" %in% names(data))) {
    stop("Missing 'c' element (count) in 'data'.")
  }
  
  if (!("t" %in% names(data))) {
    stop("Missing 't' element (time) in 'data'.")
  }
  
  if (!("k" %in% names(data))) {
    stop("Missing 'k' element (number of points) in 'data'.")
  }
  
  
  if (length(data$"c") < 4) {
    stop("Series must have at least 4 points.")
  }
  
  if (sum(which(data$"c" < 0)) > 0) {
    stop("Count must be positive or zero.")
  }
  
  if (sum(which(data$"t" <= 0)) > 0) {
    stop("Time must be strictly positive (numerical date).")
  }
  
  
  r_cand_obs <- log(data$"c"[2:data$"k"]) - log(data$"c"[1:(data$"k" - 1)])
  l_interval <- data$"t"[2:data$"k"] - data$"t"[1:(data$"k" - 1)]
  
  r_cand_obs[l_interval != 0] <- r_cand_obs[l_interval != 0] / 
    l_interval[l_interval != 0]
  
  list_start <- list()
  
  for (i in 1:nc) {
    list_start[[i]] <- list("rcand" = stats::rnorm(length(r_cand_obs), 
                                                   r_cand_obs, 1))
  }
  
  list_start
}



#' Formulation of the Bayesian model to be used by JAGS
#'
#' @description
#' The code describes a Bayesian model of the counts of one species/site over 
#' time. The model attempts to reconstitute the trend of the population size 
#' while accounting for the precision of the individual counts and the 
#' demographic potential of the species. It also implements a smoothing rule of 
#' the relative rates of increase in successive dates 
#' 
#' The code generated by this function is saved in a text file later read by 
#' the JAGS program.
#'
#' @param path a `character` string. The directory to save the file. This 
#'   directory must exist and can be an absolute or a relative path.
#'
#' @return No return value.
#' 
#' @noRd

model_formula <- function(path = ".") {
  
  if (!dir.exists(path)) {
    stop("The directory '", path, "' does not exist.")
  }
  
  
  time   <- format(Sys.time(), "%Y-%m-%d %H:%M:%S")
  
  
  header <- "\n###    Bayesian Model Formula    ###\n\n"
  header <- c(header, "# ----------------------------------")
  
  header <- c(header, "# Program: JAGS")
  header <- c(header, paste0("# Time   : ", time, ""))
  header <- c(header, "# ------------------------------")
  header <- c(header, "\n\n")
  header <- c(header, "# Data formulation\n")
  header <- c(header, "data {\n")
  header <- c(header, "\tc1 <- c[1]")
  header <- c(header, "}")
  header <- c(header, "\n\n")
  
  
  core <- ""
  
  core <- c(core, "# Model formulation\n")
  core <- c(core, "model {\n")
  
  core <- c(core, "\t# Initialisation of level of CI95%")
  core <- c(core, "\tfor (i in 1:k) {")
  core <- c(core, "\t\tsd[i]   <- (h[i] - l[i]) / 3.93")
  core <- c(core, "\t\tprec[i] <- pow(sd[i], -2)")
  core <- c(core, "\t}\n")
  
  core <- c(core, "\tfor (i in 1:(k - 1)) {")
  core <- c(core, "\t\tlint[i] <- t[i + 1] - t[i]")
  core <- c(core, "\t}\n")
  core <- c(core, "\tlinttot <- t[k] - t[1]\n")
  
  core <- c(core, "\tminN1 <- c1 / 2")
  core <- c(core, "\tmaxN1 <- c1 * 2\n")
  
  core <- c(core, "\t# Priors and constraints")
  core <- c(core, "\tN[1] ~ dunif(minN1, maxN1)\n")
  
  core <- c(core, "\t# Likelihood - State process")
  core <- c(core, "\tlogN[1]  <- log(N[1])")
  core <- c(core, "\trcand[1] ~ dnorm(0, 1)")
  core <- c(core, "\tr[1]     <- min(rcand[1], rmax)")
  core <- c(core, "\trcum[1]  <- lint[1] * r[1]")
  core <- c(core, "\tr2cum[1] <- rcum[1] * r[1]")
  core <- c(core, "\tlogN[2]  <- logN[1] + rcum[1]")
  core <- c(core, "\tN[2]     <- exp(logN[2])\n")
  
  core <- c(core, "\tfor (i in 2:(k - 1)) {")
  core <- c(core, "\t\trcand[i] ~ dnorm(r[i - 1], lability)")
  core <- c(core, "\t\tr[i]        <- min(rcand[i], rmax)")
  core <- c(core, "\t\trcum[i]     <- lint[i] * r[i]")
  core <- c(core, "\t\tr2cum[i]    <- rcum[i] * r[i]")
  core <- c(core, "\t\tlogN[i + 1] <- logN[i] + rcum[i]")
  core <- c(core, "\t\tN[i + 1]    <- exp(logN[i + 1])")
  core <- c(core, "\t}\n")
  
  core <- c(core, "\t# Likelihood - Observation process")
  core <- c(core, "\tfor (i in 1:k) {")
  core <- c(core, "\t\tc[i] ~ dnorm(N[i], prec[i])")
  core <- c(core, "\t}\n")
  
  core <- c(core, "\t# derived interesting quantities")
  core <- c(core, "\tmeanr  <- sum(rcum[]) / linttot")
  core <- c(core, "\tsdr    <- sqrt(sum(r2cum[]) / linttot - meanr * meanr)")
  core <- c(core, "\tvrrmax <- sdr / rmax\n")
  
  core <- c(core, "}\n")
  
  txt <- c(header, core)
  txt <- paste0(txt, collapse = "\n")
  
  cat(txt, file = file.path(path, "model.bug"), append = FALSE, 
      fill = TRUE)
  
  invisible(NULL)
}



#' Fit a Bayesian model to estimate population size trend
#'
#' @description
#' This function applies a Bayesian model to count series in order to infer 
#' the population trend over time. This function only works on the output of 
#' [format_data()] or [filter_series()].
#' 
#' **Important:** This function uses [R2jags::jags()] and the 
#' freeware **JAGS** (\url{https://mcmc-jags.sourceforge.io/}) must be 
#' installed.
#' 
#' There are two types of options: model options (argument `model_opts`) and 
#' MCMC options (argument `mcmc_opts`).
#' 
#' 
#' **A. Model options**
#' 
#' - a smoothing factor: the precision (the inverse of variance) of a normal 
#' distribution centered on the current relative rate of increase r from which 
#' the next candidate relative rate of increase (see below) is drawn. 
#' The highest this number, the tighter the link between successive relative 
#' rates of increase. The default `100` corresponds to a moderate link.
#' 
#' - a logical indicating whether the population growth must remain limited by 
#' the species demographic potential (provided by the argument `rmax` in 
#' [format_data()]).
#' 
#' The relative rate of increase is the change in log population size between 
#' two dates. The quantity actually being modeled is the relative rate of 
#' increase per unit of time (usually one date). This quantity reflects more 
#' directly the prevailing conditions than the population size itself, which is 
#' the reason why it has been chosen.
#' 
#' When the second model option is set to `TRUE`, the candidate rate of 
#' increase is compared to the maximum relative rate of increase 
#' (obtained when using [format_data()]) and replaced by `rmax` if greater.
#' 
#' 
#' **B. MCMC options**
#' 
#' Classical Markov chain Monte Carlo (MCMC) settings (see argument `mcmc_opts`
#' below).
#'
#' @param data a named `list`. The output of [format_data()] or 
#'   [filter_series()].
#' 
#' @param model_opts a `list` of two vectors. The model smoothing factor 
#'   (`numeric`) and a `logical` indicating if the parameter **r** must be 
#'   limited by `rmax` (`TRUE`) or not (`FALSE`). If this second parameter is 
#'   `TRUE`, the argument `rmax` cannot be `NULL` unless species are listed in 
#'   `popbayes` (in `species_info`).
#' 
#' @param mcmc_opts a `list` containing the number of iterations (`ni`), the  
#'   thinning factor (`nt`), the length of burn in (`nb`), i.e. the number of 
#'   iterations to discard at the beginning, and the number of chains (`nc`).
#'   
#' @param path a `character` string. The directory to save BUGS outputs (the 
#'   same as in [format_data()]). 
#'   
#' @export
#'
#' @return An n-element `list` (where `n` is the number of count series). Each
#'   element of the list is a BUGS output as provided by JAGS (also written 
#'   in the folder `path`).
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#' 
#' ## Get data for Alcelaphus buselaphus at Garamba only ----
#' a_buselaphus <- popbayes::filter_series(garamba_formatted, 
#'                                         location = "Garamba",
#'                                         species  = "Alcelaphus buselaphus")
#'                                         
#' \donttest{
#' ## Fit population trend (requires JAGS) ----
#' a_buselaphus_mod <- popbayes::fit_trend(a_buselaphus, path = temp_path)
#' 
#' ## Check for convergence ----
#' popbayes::diagnostic(a_buselaphus_mod, threshold = 1.1)
#' 
#' ## Plot estimated population trend ----
#' popbayes::plot_trend(series = "garamba__alcelaphus_buselaphus", 
#'                      path   = temp_path)
#' 
#' ## Plot MCMC traceplot ----
#' R2jags::traceplot(a_buselaphus_mod[[1]], ask = TRUE)
#' }

fit_trend <- function(data, model_opts = list(100, TRUE), 
                      mcmc_opts = list(ni = 50000, nt = 3, nb = 10000, nc = 2),
                      path = ".") {
  
  
  ## Check datasets ----
  
  if (!is.list(data)) {
    stop("Argument 'data' must be an output of format_data().")
  }
  
  if (!("data_converted" %in% names(data[[1]]))) {
    stop("Argument 'data' must be an output of format_data().")
  }
  
  
  ## Check path ----
  
  if (!dir.exists(path)) {
    stop("The directory '", path, "' does not exist.")
  }
  
  
  ## Check model options ----
  
  if (!is.list(model_opts)) {
    stop("Argument 'model_opts' must be a list.")
  }
  
  if (length(model_opts) != 2) {
    stop("Argument 'model_opts' must be a list of length 2.")
  }
  
  if (!is.numeric(model_opts[[1]]) || length(model_opts[[1]]) != 1) {
    stop("The first element of the list 'model_opts' must be numeric of ", 
         "length 1.")
  }
  
  if (!is.logical(model_opts[[2]]) || length(model_opts[[2]]) != 1) {
    stop("The second element of the list 'model_opts' must be TRUE or FALSE.")
  }
  
  
  ## Check MCMC options ----
  
  if (!is.list(mcmc_opts)) {
    stop("Argument 'mcmc_opts' must be a list.")
  }
  
  if (length(mcmc_opts) != 4) {
    stop("Argument 'mcmc_opts' must be a list of length 4.")
  }
  
  valid_mcmc_opts_names <- c("ni", "nt", "nb", "nc")
  valid_mcmc_opts_names_msg <- paste0(valid_mcmc_opts_names, 
                                      collapse = "' and '")
  valid_mcmc_opts_names_msg <- paste0("'", valid_mcmc_opts_names_msg, "'")
  
  if (any(!(names(mcmc_opts) %in% valid_mcmc_opts_names))) {
    stop("Invalid parameters in 'mcmc_opts'. ",
         "Required parameters are: ", valid_mcmc_opts_names_msg, ".")
  }
  
  if (!is.numeric(mcmc_opts[[1]]) || length(mcmc_opts[[1]]) != 1) {
    stop("The first element of the list 'mcmc_opts' must be numeric of ", 
         "length 1.")
  }
  
  if (!is.numeric(mcmc_opts[[2]]) || length(mcmc_opts[[2]]) != 1) {
    stop("The second element of the list 'mcmc_opts' must be numeric of ", 
         "length 1.")
  }
  
  if (!is.numeric(mcmc_opts[[3]]) || length(mcmc_opts[[3]]) != 1) {
    stop("The third element of the list 'mcmc_opts' must be numeric of ", 
         "length 1.")
  }
  
  if (!is.numeric(mcmc_opts[[4]]) || length(mcmc_opts[[4]]) != 1) {
    stop("The fourth element of the list 'mcmc_opts' must be numeric of ", 
         "length 1.")
  }
  
  
  params_to_infer <- c("N", "r", "meanr", "sdr", "vrrmax")
  
  bugs_list <- list()
  
  
  ## Loop on count series ----
  
  for (i in seq_len(length(data))) {
    
    
    ## Create sub-directory ----
    
    series_name  <- names(data)[i]
    species_path <- file.path(path, series_name)
    dir.create(species_path, showWarnings = FALSE)
    
    
    ## Model formulation ----
    
    model_formula(species_path)
    
    
    ## Prepare data ----
    
    data_sub <- data[[i]]
    
    species_rmax <- ifelse(model_opts[[2]], log(1 + data_sub$"rmax"), 10000)
    
    data_for_jags <- list(
      c        = data_sub$"data_converted"$"count_conv",
      h        = data_sub$"data_converted"$"upper_ci_conv",
      l        = data_sub$"data_converted"$"lower_ci_conv",
      t        = data_sub$"data_converted"$"date",
      k        = nrow(data_sub$"data_converted"),
      rmax     = species_rmax,
      lability = model_opts[[1]]
    )
    
    
    ## Fit population trend model ----
    
    bugs_outputs <- R2jags::jags(
      data               = data_for_jags,
      inits              = inits_plus(data_for_jags, mcmc_opts[["nc"]]),
      parameters.to.save = params_to_infer,
      model.file         = "model.bug",
      n.chains           = mcmc_opts[["nc"]],
      n.thin             = mcmc_opts[["nt"]],
      n.iter             = mcmc_opts[["ni"]],
      n.burnin           = mcmc_opts[["nb"]],
      working.directory  = species_path)
    
    invisible(file.remove(file.path(species_path, "model.bug")))
    
    bugs_list[[series_name]] <- bugs_outputs
    bugs_outputs <- bugs_list[series_name]
    
    save(bugs_outputs, file = file.path(species_path, paste0(series_name, 
                                                             "_bugs.RData")))
  }
  
  bugs_list
}



#' Import a list of BUGS outputs previously exported
#'
#' @description 
#' This function imports a list of BUGS outputs previously exported by 
#' [fit_trend()]. Users can import one, several, or all models.
#'
#' @param series a vector of `character` strings. One or several count series 
#'   names. If `NULL` (default) BUGS outputs for all count series will be 
#'   imported. Users can run [list_series()] to get the correct spelling of 
#'   count series names.
#'    
#' @param path a `character` string. The directory in which BUGS outputs have 
#'   been saved by the function [fit_trend()].
#'
#' @return An n-element `list` (where `n` is the number of count series). See
#'   [fit_trend()] for further information.
#' 
#' @export
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#' 
#' ## Select one serie ----
#' a_buselaphus <- popbayes::filter_series(garamba_formatted, 
#'                                         location = "Garamba",
#'                                         species  = "Alcelaphus buselaphus")
#' \donttest{
#' ## Fit population trends (requires JAGS) ----
#' a_buselaphus_mod <- popbayes::fit_trend(a_buselaphus, path = temp_path)
#' 
#' ## Import BUGS outputs for one count series ----
#' popbayes::read_bugs(series = "garamba__alcelaphus_buselaphus", 
#'                     path   = temp_path)
#' 
#' ## Import BUGS outputs for all count series ----
#' popbayes::read_bugs(path = temp_path)
#' }

read_bugs <- function(series = NULL, path = ".") {
    
    
  if (!dir.exists(path)) {
    stop("The directory '", path, "' does not exist.")
  }
  
  filenames <- list.files(path, recursive = TRUE, pattern = "_bugs\\.RData")
  
  if (length(filenames) == 0) {
    stop("No BUGS outputs can be found.")  
  }
  
  
  ## All available series names ----
  
  series_names <- strsplit(filenames, .Platform$"file.sep")
  series_names <- unlist(lapply(series_names, function(x) x[1]))
  
  
  if (!is.null(series)) {
    
    if (!is.character(series)) {
      stop("Argument 'series' must be a character (series name(s)).")
    }
    
    if (any(!(series %in% series_names))) {
      stop("Some count series cannot be found.")
    } 
    
    series_names <- series
  }
  
  data_bugs <- list()
  
  for (series in series_names) {
    
    data_bugs <- c(data_bugs, get(load(file.path(path, series, 
                                                 paste0(series, 
                                                        "_bugs.RData")))))
  }
  
  data_bugs
}



#' Extract estimated parameters from a list of BUGS outputs
#'
#' @description
#' From the output of the function [fit_trend()] (or [read_bugs()]), this 
#' function extracts estimated parameters into a `data.frame`.
#' 
#' The resulting `data.frame` has no particular use in `popbayes` but it can be
#' useful for users.
#'
#' @param data a named `list` of BUGS outputs. The output of [fit_trend()] or
#'   [read_bugs()]
#'
#' @return A `data.frame`.
#' 
#' @export
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#'                                         
#' ## Select one serie ----
#' a_buselaphus <- popbayes::filter_series(garamba_formatted, 
#'                                         location = "Garamba",
#'                                         species  = "Alcelaphus buselaphus")
#' \donttest{
#' ## Fit population trends (requires JAGS) ----
#' a_buselaphus_mod <- popbayes::fit_trend(a_buselaphus, path = temp_path)
#' 
#' ## Import BUGS outputs for one count series ----
#' bugs <- popbayes::read_bugs(series = "garamba__alcelaphus_buselaphus", 
#'                             path   = temp_path)
#' 
#' ## Extract estimated parameters ----
#' popbayes::bugs_to_df(bugs)
#' }

bugs_to_df <- function(data) {
  
  if (!is.list(data)) {
    stop("Argument 'data' must be an output of fit_trend().")
  }
  
  if (!("BUGSoutput" %in% names(data[[1]]))) {
    stop("Argument 'data' must be an output of fit_trend().")
  }
  
  
  series_names <- names(data)
  
  bugs <- lapply(data, function(x) x[["BUGSoutput"]]$"summary")
  bugs <- lapply(seq_len(length(series_names)), function(x, y, z) {
    bug <- cbind(z[x], rownames(y[[x]]), as.data.frame(y[[x]]))
    colnames(bug)[1:2] <- c("series", "parameter")
    rownames(bug) <- NULL
    bug
  }, y = bugs, z = series_names)
  
  bugs <- do.call(rbind.data.frame, bugs)
  rownames(bugs) <- NULL
  
  bugs
}



#' Check if a BUGS model has converged
#'
#' @description
#' From the output of the function [fit_trend()] (or [read_bugs()]), this 
#' function checks if the estimation of all parameters of one (or several) 
#' BUGS model has converged. This diagnostic is performed by comparing the 
#' `Rhat` value of each parameter to a `threshold` (default is `1.1`). If some 
#' `Rhat` values are greater than this threshold (no convergence), a message
#' listing problematic models is displayed.
#'
#' @param data a named `list` of BUGS outputs. The output of [fit_trend()] or 
#'   [read_bugs()].
#'   
#' @param threshold a `numeric`.
#'
#' @return No return value.
#' 
#' @export
#'
#' @examples
#' ## Load Garamba raw dataset ----
#' file_path <- system.file("extdata", "garamba_survey.csv", 
#'                          package = "popbayes")
#'                          
#' garamba <- read.csv(file = file_path)
#' 
#' ## Create temporary folder ----
#' temp_path <- tempdir()
#' 
#' ## Format dataset ----
#' garamba_formatted <- popbayes::format_data(
#'   data              = garamba, 
#'   path              = temp_path,
#'   field_method      = "field_method",
#'   pref_field_method = "pref_field_method",
#'   conversion_A2G    = "conversion_A2G",
#'   rmax              = "rmax")
#'                                         
#' ## Select one serie ----
#' a_buselaphus <- popbayes::filter_series(garamba_formatted, 
#'                                         location = "Garamba",
#'                                         species  = "Alcelaphus buselaphus")
#' \donttest{
#' ## Fit population trends (requires JAGS) ----
#' a_buselaphus_mod <- popbayes::fit_trend(a_buselaphus, path = temp_path)
#' 
#' ## Check for convergence ----
#' popbayes::diagnostic(a_buselaphus_mod)
#' }

diagnostic <- function(data, threshold = 1.1) {
  
  if (!is.list(data)) {
    stop("Argument 'data' must be an output of fit_trend().")
  }
  
  if (!("BUGSoutput" %in% names(data[[1]]))) {
    stop("Argument 'data' must be an output of fit_trend().")
  }
  
  
  series_names <- names(data)
  
  bugs <- lapply(data, function(x) x[["BUGSoutput"]]$"summary")
  bugs <- lapply(bugs, function(x) {
    length(which(x[ , "Rhat"] > threshold))
  })
  
  bugs <- unlist(bugs)
  
  pos <- which(bugs > 0)
  
  if (length(pos)) {
    
    no_converged <- names(bugs)[pos]
    cli::cli_alert_danger(c(
      "Some parameters have not converged for the following count series: ",
      "{.val {no_converged}}."
    ))
    
  } else {
    
    cli::cli_alert_success("All models have converged.")
  }
  
  invisible(NULL)
}

---
title: "Get species information"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get species information}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>"
)
```



## Calculating rmax

The demographic potential of a species is limited. The intrinsic rate of 
increase $rmax$ is the maximum increase in log population size that a species 
can attain in a year. According to Sinclair (2003), it is related to the body 
mass of adult females $W$  by the formula:


$$ rmax = 1.375 \times W^{-0.315} $$


Body masses are found in the literature in publications such as Kingdon & 
Hoffman (2013), Cornelis _et al._ (2014), Illius & Gordon (1992), 
Sinclair (1996), Suraud _et al._ (2012), or Foley & Faust (2010). 
Alternatively, $rmax$ can be obtained from specific demographic analyses. In the 
following table, we have listed the $rmax$ values obtained with one or the other 
method giving precedence to specific analyses when available.  



```{r 'rmax'}
species_name <- c("Impala", "Tiang", "Blue wildebeest", "Roan", "Buffalo", 
                  "Eland", "Giraffe", "Elephant")

adult_female_body_mass <- c(55, 127, 230, 250, 400, 450, 702, 2873)

species <- data.frame(adult_female_body_mass, row.names = species_name)

species$rmax <- 1.375 * adult_female_body_mass ^ (-0.315)
species["Eland", "rmax"]    <- 0.150                 # Sinclair (1996)
species["Elephant", "rmax"] <- 0.07                  # Foley & Faust (2010)
species["Giraffe", "rmax"]  <- 0.125                 # Suraud et al. (2012)

species
```



## Building conversion table

Aerial and ground counts are not directly comparable because some species are 
better detected from the ground and others from the air. It is generally 
considered that small light species are better detected from the ground while 
large dark species are better detected from the air. We took advantage of series 
of counts carried out in parallel using the two field methods to calculate 
conversion factors to be applied to aerial counts to obtain ground count 
equivalents. This permits reconciling the two types of counts within a mixed 
series. We also specify the preferred field method for each category of species.



```{r 'conversion_fact'}
categories <- c("Medium light and brown species (20-150kg)",
                "Large light and brown species (>150kg)",
                "Large dark (>150kg)", "Giraffe", "Elephant")

short_names        <- c("MLB", "LLB", "LD", "Giraffe", "Elephant")
conversion_facts   <- c(6.747, 2.302, 0.561, 3.011, 0.659)
pref_field_methods <- c("G", "G", "A", "A", "A")

category_info <- data.frame("category"          = categories,
                            "acronym"           = short_names,
                            "conversion_fact"   = conversion_facts, 
                            "pref_field_method" = pref_field_methods)
category_info
```



## Categorizing species

Here we relate the species to a color/size category.


```{r}
species$"category" <- c("MLB", "MLB", "LLB", "LLB", "LD", "LLB", "Giraffe", 
                        "Elephant")
species
```



<br />



## References


- Cornelis D _et al._ (2014) Species account: African buffalo (_Syncerus caffer_).
In: _Ecology, Evolution and Behaviour of Wild Cattle: Implications for Conservation_
(Eds M Melletti & J Burton). Cambridge University Press, Cambridge.
DOI: [10.1017/CBO9781139568098](https://doi.org/10.1017/CBO9781139568098).

- Foley CAH & Faust LJ (2010) Rapid population growth in an elephant _Loxodonta africana_ 
population recovering from poaching in Tarangire National Park, Tanzania.
_Oryx_, **44**, 205-212.
DOI: [10.1017/S0030605309990706](https://doi.org/10.1017/S0030605309990706).

- Illius AW & Gordon IJ (1992) Modelling the nutritional ecology of ungulate 
herbivores: evolution of body size and competitive interactions. 
_Oecologia_, **89**, 428-434.
DOI: [10.1017/S0030605309990706](https://doi.org/10.1007/BF00317422).

- Kingdon J & Hoffman M (2013) 
_Mammals of Africa. Volume VI: Pigs, Hippopotamuses, Chevrotain, Giraffes, Deer and Bovids_.
Bloomsbury Publishing, London, United Kingdom, 680 pp.

- Sinclair ARE (1996) Mammal populations: fluctuation, regulation, life history 
theory, and their implications for conservation. 
In: _Frontiers of population ecology_ (Eds RB Floyd & AW Sheppard), 
pp. 127–154. CSIRO: Melbourne, Australia.

- Sinclair (2003) Mammal population regulation, keystone processes and
ecosystem dynamics.
_Philosophical Transactions: Biological Sciences_, **358**, 1729-1740.
DOI: [10.1098/rstb.2003.1359](https://doi.org/10.1098/rstb.2003.1359).

- Suraud JP _et al._ (2012)
Higher than expected growth rate of the endangered West African giraffe 
_Giraffa camelopardalis peralta_: a successful human–wildlife cohabitation. 
_Oryx_, **46**, 577-583.
DOI: [10.1017/S0030605311000639](https://doi.org/10.1017/S0030605311000639).

---
title: "Get started"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse   = TRUE,
  comment    = "#>",
  fig.width  = 7,
  fig.height = 7,
  out.width  = "100%",
  dpi        = 300
)
```

```{r 'setup', echo = FALSE}
library("popbayes")
```

The goal of the R package `popbayes` is to fit population trajectories over time 
from counts of individuals collected at various dates and with a variety of 
methods. It does so under a Bayesian framework where the primary quantity being 
modeled is the rate of increase between successive years (or any other time units 
for that matter, the one used for date). The package can deal with multiple 
species and multiple locations presented in a single data set, but each 
**count series** made of the counts relative to one species at one location 
will be processed independently.

The strength of `popbayes` is to handle, in a single series, counts collected 
under different types of surveys (aerial vs ground surveys), and estimated by 
different census methods (total counts, sampling counts, and even guesstimates 
[i.e. expert estimates]).



<br />

**Before using this package, users need to install the freeware 
[JAGS](https://mcmc-jags.sourceforge.io/).**

<br />


The workflow of `popbayes` consists in three main steps:

1. Formatting data (`format_data()`)
2. Fitting trends (`fit_trend()`)
3. Visualizing results (`plot_trend()`)

The package also provides a lot of functions to handle individual count series 
and model outputs. The following figure shows a more complete usage of the 
package.


<br />


```{r, echo = FALSE, out.width = "100%", fig.cap = "Framework of `popbayes`", fig.align = 'center'}
knitr::include_graphics("docs/popbayes-diagram.png")
```


<br />



## The Garamba dataset



The package `popbayes` comes with an example dataset: `garamba`. It contains 
counts of individuals from 10 African mammal species surveyed in the Garamba 
National Park (Democratic Republic of the Congo) from 1976 to 2017.


<br />

```{r 'load-data'}
## Define filename path ----
file_path <- system.file("extdata", "garamba_survey.csv", package = "popbayes")

## Read CSV file ----
garamba <- read.csv(file = file_path)
```

<br />

```{r 'print-data', echo = FALSE}
knitr::kable(head(garamba, 20), caption = "The Garamba dataset (first 20 rows)", 
             align = c("c", "l", "c", "c", "c", "r", "r", "r", "c", "r", "r"))
```


<br />


This dataset has a typical structure with a location field (`location`), a 
species name field (`species`), a date field (`date`), and a count
field (`count`).  

<br />

### Statistical method

In addition to the fields `location`, `species`, `date`, and `count`, a fourth 
field is **mandatory**: `stat_method`. This field specifies the 
census method that produced the count. It can be `T` for a total 
count, `X` for a guesstimate (i.e. expert estimate), or `S` for a sampling count.

To be usable by the Bayesian model, individual counts are to be accompanied 
by information on precision in the form of a 95% confidence interval. If counts 
are :

- `T` or `X`, a confidence interval will be computed automatically by the 
function `format_data()` according respectively to the following formulas:

$$ CI_{(T)} = [\ 0.95 \times count\ ; 1.20 \times count\ ] $$
$$ CI_{(X)} = [\ 0.80 \times count\ ; 1.20 \times count\ ] $$

- `S`, users **need to supply** a measure of precision. Precision 
is preferably provided in the form of a 95% CI by means of two fields: `lower_ci` 
and `upper_ci` (as in the `garamba` dataset). Alternatively, it may also be given 
in the form of a standard deviation (`sd`), a variance (`var`), or a 
coefficient of variation (`cv`). Note that precision metrics can be different 
between counts. For instance, some `S` counts may have an `sd` value and others 
`lower_ci` and `upper_ci`. In that case, three precision columns would be required
(`lower_ci`, `upper_ci`, and `sd`). An `S` count with no measure of precision will 
be detected as an anomaly by `format_data()` by default. The option `na.rm = TRUE`
may be used to automatically remove such counts from the series. If it is 
desirable to maintain such counts in the count series, we suggest to enter a 
value for the coefficient of variation, e.g. the average coefficient of variation 
of the other counts in the series.   


<br />

### Field method

Another **optional** column, `field_method`, may be provided. It refers to the 
type of survey used to collect data. This can be `A` for aerial survey or `G` 
for ground survey. This column becomes mandatory as soon as both field methods 
are present in a series.

The detectability of a species is indeed strongly dependent on the survey method 
and each species has its own _preferred field method_, the one that is assumed 
to provide estimates closer to the truth. So, even if a series is homogeneous 
relative to the `field method`, it is recommended to provide the column 
`field_method` if counts have been collected under the not preferred 
field method. That will force conversion towards the _preferred field method_.


<br />

### Count conversion

The function `format_data()` will convert counts (and 95% CI bounds) into their 
equivalent in the preferred field method for the species. To this aim, two 
pieces of information are required :

- `pref_field_method`: the preferred field method for the species (`A` or `G`);
- `conversion_A2G`: the multiplicative factor used to convert an aerial count 
into an equivalent ground count.

The package `popbayes` provides the `species_info` dataset, which contains
these two pieces of information for 15 African mammal species. 

<br />

```{r 'load-data-2'}
data("species_info")
```

<br />

```{r 'print-data-2', echo = FALSE}
knitr::kable(species_info[ , c(3, 6, 7, 8, 9)], 
             caption = "Species with count conversion information in popbayes", 
             align = c("l", "c", "c", "r", "r"))
```

<br />

If users work only with species in this table, the package `popbayes` can 
automatically retrieve the values of `pref_field_method` and `conversion_A2G` 
from the `species_info` data set. But for other species, users 
**need to supply** the information themselves when running `format_data()`. 
These values may be provided as additional fields in the count data set. Care 
must then be taken that the same value is consistently repeated for each count 
of the same species. For users with sufficient command of R, we recommend rather 
to create an independent additional table similar to `species_info` and to 
pass it to the function `format_data()` as the data frame argument `info`. 

**Note**: Currently `format_data()` takes its information for count conversion 
from **one source only** with priority given to `info`, then to additional 
fields in data (if `info` is not provided), and eventually to the 
`species_info` table of the package (when the other two sources are lacking). 
That means that the source with the highest priority must be complete with 
respect to the species present in data, as it will be used exclusively to any 
other source. If, say, you use `info`, you cannot expect `format_data()` to 
retrieve conversion information for a species undocumented in `info` from the 
`species_info` table of the package. However, you can easily construct `info` 
from a copy of `species_info`, which additionally provides a ready template. 
It suffices to add any species not already in `species_info` as shown below.

Let's assume that, in addition to other species present in the package 
`species_info` table, we have counts of _Taurotragus oryx_ and 
_Taurotragus derbianus_. We can construct `info` as follows. 


<br />


```{r 'create-info'}
## Extract the relevant columns of the package table "species_info" ----
info_from_package <- species_info[ , c("species", "pref_field_method", "conversion_A2G", "rmax")]

## Add the new species ----
new_conversion_info <- data.frame("species"           = c("Taurotragus oryx","Taurotragus derbianus"),
                                  "pref_field_method" = "G",
                                  "conversion_A2G"    = 2.302,
                                  "rmax"              = 0.1500)

## Append the new species ----
info <- rbind(info_from_package, new_conversion_info)
info
```

<br />

If you do not have conversion information of your own for a new species, you 
can rely on the conversion information of species with similar characteristics
(for example the two _Taurotragus_ species belong to the category LLB). 
The package `popbayes` distinguishes five categories of species:

- **MLB**: Medium-sized Light and Brown species (20-150kg)
- **LLB**: Large Light and Brown species (>150kg)
- **LD**: Large Dark (>150kg)
- **Elephant**
- **Giraffe**

The field `category` of the `species_info` table indicates which species 
belong to each.


<br />

### Relative rate of increase

The demographic potential of a species is limited. The intrinsic rate of 
increase (called `rmax`) is the maximum increase in log population size that a 
species can attain in a year.

We strongly recommend using the `rmax` values while estimating population trend
to limit yearly population growth estimated by the model (the default).

<br />

As for `pref_field_method` and `conversion_A2G`, `rmax` values (specific to a
species) can be provided in an additional field of the count dataset (`garamba`),
as additional field of the `info` data frame, or internally can be retrieved 
from the internal dataset of `popbayes`.

<br />

**How to find the species `rmax` value?**

According to Sinclair (2003), `rmax` is related to the body mass of adult 
females `W`  by the formula:

$$ rmax = 1.375 \times W^{-0.315} $$

Body masses are found in the literature in publications such as Kingdon & 
Hoffman (2013), Cornelis _et al._ (2014), Illius & Gordon (1992), 
Sinclair (1996), Suraud _et al._ (2012), or Foley & Faust (2010). 

If you know the body mass of adult females of the species, you can compute the
`rmax` value with the function `w_to_rmax()`.

<br />

Alternatively, `rmax` can be obtained from previous demographic analyses.

<br />

**Important note**: The intrinsic rate of increase refers to a change over one 
year. If a different time unit is used for the dates (say a month), the rmax to
provide must be adapted (here divided by 12). The **rmax values in popbayes cannot 
be used for time units other than one year**.

<br />


## Checking data


The first thing that the function `format_data()` does is to check the validity 
of the content of the different fields of the count data set. Here we will 
explore our data to avoid errors when using the function `format_data()`.

In particular, we need to check `location` and `species` spelling, `date` and 
`count` field format, and the `stat_method` and `field_method` categories.


<br />

**Check `location` field**

```{r 'check-location'}
unique(garamba$"location")

sum(is.na(garamba$"location"))   # Are there any missing values?
```

Field **`location`** can be either a `character` or a `factor`. It **cannot** 
contain any `NA` values.


<br />

**Check `species` field**

```{r 'check-species'}
unique(garamba$"species")

sum(is.na(garamba$"species"))   # Are there any missing values?

## Are there species absent from the 'species_info' popbayes dataset?
garamba_species <- unique(garamba$"species")
garamba_species[which(!(garamba_species %in% species_info$"species"))]
```

Field **`species`** can be either a `character` or a `factor`. It **cannot** 
contain any `NA` values.


<br />

**Check `date` field**

```{r 'check-date'}
is.numeric(garamba$"date")     # Are dates in a numerical format?

sum(is.na(garamba$"date"))     # Are there any missing values?

range(garamba$"date")          # What is the temporal extent?
```

Field **`date`** must be a `numeric`. It **cannot** contain any `NA` values. 
This said, the time unit is arbitrary, and fractional values of years (or 
another unit) are allowed. As long as numeric values are entered, the package 
will work.

<br />

On the other hand, if you have a date format (e.g. '2021/05/19'), you need to 
convert it to a numeric format. For instance:

```{r 'convert-date'}
## Convert a character to a date object ----
x <- as.Date("2021/05/19")
x

## Convert a date to a numeric (number of days since 1970/01/01) ----
x <- as.numeric(x)
x

## Check ----
as.Date(x, origin = as.Date("1970/01/01"))
```

Other methods exist to convert a `date` to a `numeric` format. You may prefer 
computing the number of days since the first date of your survey. It's up to you.


<br />

**Check `count` field**

```{r 'check-counts'}
is.numeric(garamba$"count")   # Are counts in a numerical format?

range(garamba$"count")        # What is the range of values?

sum(is.na(garamba$"count"))   # Are there any missing values?
```

Field **`count`** must be a **positive** `numeric` (zero counts are allowed).
`NA` counts cannot be used for fitting trends. The `format_data()` function 
(see below) has an option for dropping them.


<br />

**Check `stat_method` field**

```{r 'check-stat'}
unique(garamba$"stat_method")

sum(is.na(garamba$"stat_method"))   # Are there any missing values?
```

Field **`stat_method`** can be either a `character` or a `factor`. It **must**
contain only `T`, `X`, or `S` categories and **cannot** contain any `NA` values.


<br />

**Check `field_method` field**

```{r 'check-field'}
unique(garamba$"field_method")

sum(is.na(garamba$"field_method"))   # Are there any missing values?
```

Field **`field_method`** can be either a `character` or a `factor`. It **must**
contain only `A`, or `G` categories and **cannot** contain any `NA` values.



<br />



## Formatting data

This first `popbayes` function to use is `format_data()`. This function 
provides an easy way to get individual count series ready to be analyzed by 
the package. It must be **used prior to** all other functions.


<br />

First let's define the path (relative or absolute) to save objects/results, 
namely the formatted count series that can be extracted from the data set.

```{r 'define-path-1', eval = TRUE, echo = FALSE}
path <- tempdir()
```

```{r 'define-path-2', eval = FALSE, echo = TRUE}
path <- "the_folder_to_store_outputs"
```


<br />

The function `format_data()` has many arguments to provide the names of the 
columns in the user's dataset that contain `location`, `species`, `lower_ci`, 
etc. By default column names are the same as in the Garamba dataset. If your 
location field, say, is "site", you'll need to use the argument `location` as 
follows: `location = "site"`.


<br />

```{r 'format-data'}
garamba_formatted <- popbayes::format_data(data              = garamba, 
                                           path              = path,
                                           field_method      = "field_method",
                                           pref_field_method = "pref_field_method",
                                           conversion_A2G    = "conversion_A2G",
                                           rmax              = "rmax")
```

<br />


As said above, if you have to add your own count conversion data, you need 
specify the names of columns for the preferred field method, the conversion 
factor, and rmax as this:
`pref_field_method = "column_with_preferred_field_method"`, 
`conversion_A2G = "column_with_conversion_A2Gor"`, 
`rmax = "column_with_conversion_rmax"`,
or alternatively use the argument `info`:
`info = "dataframe_with_conversion_info"`.

<br />

Let's explore the output.

```{r 'explore-series'}
## Class of the object ----
class(garamba_formatted)

## Number of elements (i.e. number of count series) ----
length(garamba_formatted)

## Get series names ----
popbayes::list_series(path)
```

<br />

Let's work with the count series `"garamba__alcelaphus_buselaphus"`. We can use
the function `filter_series()`.

```{r 'filter-series'}
## Retrieve series by species and location ----
a_buselaphus <- popbayes::filter_series(data     = garamba_formatted, 
                                        species  = "Alcelaphus buselaphus",
                                        location = "Garamba")
```

<br />

Let's display the series content.

```{r 'print-series'}
print(a_buselaphus)
```

<br />

The first elements of the list provide a summary of the count series.

If we compare the two last data frames (`data_original` and `data_converted`), 
they are not identical. The function `format_data()` has **1)** computed 
95% CI boundaries for total counts (coded `T` in the column `stat_method`), and 
**2)** converted all counts (and CI boundaries) to their equivalent in the 
preferred field method (from `A` to `G`) by applying the conversion factor of 
`2.302`.

The Bayesian model will use counts and precision measures from the 
`data_converted` data frame.

<br />

Before fitting the population size trend we can visualize the count series with 
`plot_series()`.

<br />

```{r 'plot-series-1', fig.width=12, fig.height=6, out.width="100%"}
popbayes::plot_series("garamba__alcelaphus_buselaphus", path = path)
```

<br />



The function `format_data()` has also exported the count series as `.RData` 
files in the `path` folder where they have been dispatched into sub-folders, 
one per series.


```{r 'list-folder-1', echo = TRUE, eval = FALSE}
list.files(path, recursive = TRUE)
```

```{r 'list-folder-2', echo = FALSE, eval = TRUE}
list.files(path, recursive = TRUE, pattern = "^garamba__")
```


<br />

These `*_data.RData` files (count series) can be imported later by running the 
function `read_series()`.


```{r 'read-series', eval = FALSE}
a_buselaphus <- popbayes::read_series("garamba__alcelaphus_buselaphus", path = path)
```


<br />



## Fitting trend


The function `fit_trend()` fits population trajectories over time 
from counts of individuals formatted by `format_data()`. It does so under a 
Bayesian framework where the primary quantity being modeled is the annual rate 
of increase (more generally, the rate of increase per the time unit used for dates).

This function only works on the output of `format_data()` (or `filter_series()`).

Here is the default usage of the function `fit_trend()`:

<br />

```{r 'run-jags', eval = FALSE}
a_buselaphus_bugs <- popbayes::fit_trend(a_buselaphus, path = path)
```

<br />

The function returns an n-element list (where n is the number of count series). 
Each element of the list is a BUGS output as provided by JAGS. It has also 
exported these BUGS outputs as `.RData` files in the `path` folder where they 
have been dispatched into sub-folders, one per series.


<br />

These `*_bugs.RData` files (BUGS outputs) can be imported later by running the 
function `read_bugs()`.


```{r 'read-bugs', eval = FALSE}
a_buselaphus_bugs <- popbayes::read_bugs("garamba__alcelaphus_buselaphus", path = path)
```


<br />

The function `diagnostic()` allows to check if estimation of all parameters of 
the model has converged. This diagnostic is performed by comparing the 
`Rhat` value of each parameter to a `threshold` (default is `1.1`).

<br />

```{r 'diagnostic', eval = FALSE}
popbayes::diagnostic(a_buselaphus_bugs)
#> All models have converged.
```

In case convergence was not reached for some series, we suggest rerunning 
`fit_trend()` on these series after increasing the number of iterations 
(**ni**) and possibly the number of initial iterations discarded (**nb**) from 
their respective defaults of 50,000 and 10,000. For example:

```{r 're-run-jags', eval = FALSE}
a_buselaphus_bugs <- popbayes::fit_trend(a_buselaphus, path = path, ni = 100000, nb = 20000)
```

This process may be repeated with increasing values of **ni** and **nb** until 
convergence is eventually reached.


<br />

Finally we can use the function `plot_trend()` to visualize model predictions and
estimated yearly relative growth rates.

<br />

```{r 'plot-trend-1', fig.width=12, fig.height=6, out.width="100%", eval = FALSE, echo = TRUE}
popbayes::plot_trend("garamba__alcelaphus_buselaphus", path = path)
```

```{r, echo = FALSE, out.width = "100%", fig.align = 'center'}
knitr::include_graphics("docs/garamba__alcelaphus_buselaphus.png")
```



<br />



## References


- Cornelis D _et al._ (2014) Species account: African buffalo (_Syncerus caffer_).
In: _Ecology, Evolution and Behaviour of Wild Cattle: Implications for Conservation_
(Eds M Melletti & J Burton). Cambridge University Press, Cambridge.
DOI: [10.1017/CBO9781139568098](https://doi.org/10.1017/CBO9781139568098).

- Foley CAH & Faust LJ (2010) Rapid population growth in an elephant _Loxodonta africana_ 
population recovering from poaching in Tarangire National Park, Tanzania.
_Oryx_, **44**, 205-212.
DOI: [10.1017/S0030605309990706](https://doi.org/10.1017/S0030605309990706).

- Illius AW & Gordon IJ (1992) Modelling the nutritional ecology of ungulate 
herbivores: evolution of body size and competitive interactions. 
_Oecologia_, **89**, 428-434.
DOI: [10.1017/S0030605309990706](https://doi.org/10.1007/BF00317422).

- Kingdon J & Hoffman M (2013) 
_Mammals of Africa. Volume VI: Pigs, Hippopotamuses, Chevrotain, Giraffes, Deer and Bovids_.
Bloomsbury Publishing, London, United Kingdom, 680 pp.

- Sinclair ARE (1996) Mammal populations: fluctuation, regulation, life history 
theory, and their implications for conservation. 
In: _Frontiers of population ecology_ (Eds RB Floyd & AW Sheppard), 
pp. 127-154. CSIRO: Melbourne, Australia.

- Sinclair ARE (2003) Mammal population regulation, keystone processes and
ecosystem dynamics.
_Philosophical Transactions: Biological Sciences_, **358**, 1729-1740.
DOI: [10.1098/rstb.2003.1359](https://doi.org/10.1098/rstb.2003.1359).

- Suraud JP _et al._ (2012)
Higher than expected growth rate of the endangered West African giraffe 
_Giraffa camelopardalis peralta_: a successful human-wildlife cohabitation. 
_Oryx_, **46**, 577-583.
DOI: [10.1017/S0030605311000639](https://doi.org/10.1017/S0030605311000639).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bugs-funs.R
\name{fit_trend}
\alias{fit_trend}
\title{Fit a Bayesian model to estimate population size trend}
\usage{
fit_trend(
  data,
  model_opts = list(100, TRUE),
  mcmc_opts = list(ni = 50000, nt = 3, nb = 10000, nc = 2),
  path = "."
)
}
\arguments{
\item{data}{a named \code{list}. The output of \code{\link[=format_data]{format_data()}} or
\code{\link[=filter_series]{filter_series()}}.}

\item{model_opts}{a \code{list} of two vectors. The model smoothing factor
(\code{numeric}) and a \code{logical} indicating if the parameter \strong{r} must be
limited by \code{rmax} (\code{TRUE}) or not (\code{FALSE}). If this second parameter is
\code{TRUE}, the argument \code{rmax} cannot be \code{NULL} unless species are listed in
\code{popbayes} (in \code{species_info}).}

\item{mcmc_opts}{a \code{list} containing the number of iterations (\code{ni}), the
thinning factor (\code{nt}), the length of burn in (\code{nb}), i.e. the number of
iterations to discard at the beginning, and the number of chains (\code{nc}).}

\item{path}{a \code{character} string. The directory to save BUGS outputs (the
same as in \code{\link[=format_data]{format_data()}}).}
}
\value{
An n-element \code{list} (where \code{n} is the number of count series). Each
element of the list is a BUGS output as provided by JAGS (also written
in the folder \code{path}).
}
\description{
This function applies a Bayesian model to count series in order to infer
the population trend over time. This function only works on the output of
\code{\link[=format_data]{format_data()}} or \code{\link[=filter_series]{filter_series()}}.

\strong{Important:} This function uses \code{\link[R2jags:jags]{R2jags::jags()}} and the
freeware \strong{JAGS} (\url{https://mcmc-jags.sourceforge.io/}) must be
installed.

There are two types of options: model options (argument \code{model_opts}) and
MCMC options (argument \code{mcmc_opts}).

\strong{A. Model options}
\itemize{
\item a smoothing factor: the precision (the inverse of variance) of a normal
distribution centered on the current relative rate of increase r from which
the next candidate relative rate of increase (see below) is drawn.
The highest this number, the tighter the link between successive relative
rates of increase. The default \code{100} corresponds to a moderate link.
\item a logical indicating whether the population growth must remain limited by
the species demographic potential (provided by the argument \code{rmax} in
\code{\link[=format_data]{format_data()}}).
}

The relative rate of increase is the change in log population size between
two dates. The quantity actually being modeled is the relative rate of
increase per unit of time (usually one date). This quantity reflects more
directly the prevailing conditions than the population size itself, which is
the reason why it has been chosen.

When the second model option is set to \code{TRUE}, the candidate rate of
increase is compared to the maximum relative rate of increase
(obtained when using \code{\link[=format_data]{format_data()}}) and replaced by \code{rmax} if greater.

\strong{B. MCMC options}

Classical Markov chain Monte Carlo (MCMC) settings (see argument \code{mcmc_opts}
below).
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")

## Get data for Alcelaphus buselaphus at Garamba only ----
a_buselaphus <- popbayes::filter_series(garamba_formatted, 
                                        location = "Garamba",
                                        species  = "Alcelaphus buselaphus")
                                        
\donttest{
## Fit population trend (requires JAGS) ----
a_buselaphus_mod <- popbayes::fit_trend(a_buselaphus, path = temp_path)

## Check for convergence ----
popbayes::diagnostic(a_buselaphus_mod, threshold = 1.1)

## Plot estimated population trend ----
popbayes::plot_trend(series = "garamba__alcelaphus_buselaphus", 
                     path   = temp_path)

## Plot MCMC traceplot ----
R2jags::traceplot(a_buselaphus_mod[[1]], ask = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bugs-funs.R
\name{diagnostic}
\alias{diagnostic}
\title{Check if a BUGS model has converged}
\usage{
diagnostic(data, threshold = 1.1)
}
\arguments{
\item{data}{a named \code{list} of BUGS outputs. The output of \code{\link[=fit_trend]{fit_trend()}} or
\code{\link[=read_bugs]{read_bugs()}}.}

\item{threshold}{a \code{numeric}.}
}
\value{
No return value.
}
\description{
From the output of the function \code{\link[=fit_trend]{fit_trend()}} (or \code{\link[=read_bugs]{read_bugs()}}), this
function checks if the estimation of all parameters of one (or several)
BUGS model has converged. This diagnostic is performed by comparing the
\code{Rhat} value of each parameter to a \code{threshold} (default is \code{1.1}). If some
\code{Rhat} values are greater than this threshold (no convergence), a message
listing problematic models is displayed.
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")
                                        
## Select one serie ----
a_buselaphus <- popbayes::filter_series(garamba_formatted, 
                                        location = "Garamba",
                                        species  = "Alcelaphus buselaphus")
\donttest{
## Fit population trends (requires JAGS) ----
a_buselaphus_mod <- popbayes::fit_trend(a_buselaphus, path = temp_path)

## Check for convergence ----
popbayes::diagnostic(a_buselaphus_mod)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/series-funs.R
\name{list_series}
\alias{list_series}
\title{Retrieve the count series names}
\usage{
list_series(path = ".")
}
\arguments{
\item{path}{a \code{character} string. The directory in which count series have
been saved by the function \code{\link[=format_data]{format_data()}}.}
}
\value{
A vector of count series names (\code{character} strings).
}
\description{
This function retrieves the count series names generated by
the function \code{\link[=format_data]{format_data()}}.
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")

## Retrieve count series names ----
popbayes::list_series(path = temp_path)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/popbayes-package.R
\docType{package}
\name{popbayes-package}
\alias{popbayes}
\alias{popbayes-package}
\title{popbayes: Bayesian Model to Estimate Population Trends from Counts Series}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Infers the trends of one or several animal populations over time from series of counts. It does so by accounting for count precision (provided or inferred based on expert knowledge, e.g. guesstimates), smoothing the population rate of increase over time, and accounting for the maximum demographic potential of species. Inference is carried out in a Bayesian framework. This work is part of the FRB-CESAB working group AfroBioDrivers \url{https://www.fondationbiodiversite.fr/en/the-frb-in-action/programs-and-projects/le-cesab/afrobiodrivers/}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://frbcesab.github.io/popbayes/}
  \item \url{https://github.com/frbcesab/popbayes}
  \item Report bugs at \url{https://github.com/frbcesab/popbayes/issues}
}

}
\author{
\strong{Maintainer}: Nicolas Casajus \email{nicolas.casajus@fondationbiodiversite.fr} (\href{https://orcid.org/0000-0002-5537-5294}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Roger Pradel \email{roger.pradel@cefe.cnrs.fr} (\href{https://orcid.org/0000-0002-2684-9251}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-funs.R
\name{plot_series}
\alias{plot_series}
\title{Plot original vs converted counts}
\usage{
plot_series(series, title = TRUE, path = ".", path_fig = ".", save = FALSE)
}
\arguments{
\item{series}{a \code{character} string. The count series names (can be
retrieved by running \code{\link[=list_series]{list_series()}}).}

\item{title}{a \code{logical}. If \code{TRUE} (default) a title (series name) is
added.}

\item{path}{a \code{character} string. The directory in which count series have
been saved by the function \code{\link[=format_data]{format_data()}}.}

\item{path_fig}{a \code{character} string. The directory where to save the plot
(if \code{save = TRUE}). This directory must exist and can be an absolute or a
relative path.}

\item{save}{a \code{logical}. If \code{TRUE} (default is \code{FALSE}) the plot is saved in
\code{path_fig}.}
}
\value{
No return value.
}
\description{
This function plots a panel of two graphics for one count series
(previously generated by \code{\link[=format_data]{format_data()}}):
\itemize{
\item on the left side, scatter plot overlapping original (black points) and
converted counts (grey points);
\item on the right side, scatter plot of converted counts with boundaries of the
95\% confident interval.
}
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")

## Get series names ----
popbayes::list_series(path = temp_path)

## Plot for Alcelaphus buselaphus at Garamba ----
popbayes::plot_series("garamba__alcelaphus_buselaphus", path = temp_path)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/series-funs.R
\name{series_to_df}
\alias{series_to_df}
\title{Extract original/converted count series data from a list}
\usage{
series_to_df(data, converted = TRUE)
}
\arguments{
\item{data}{a named \code{list}. The output of \code{\link[=format_data]{format_data()}} or
\code{\link[=filter_series]{filter_series()}}.}

\item{converted}{a \code{logical}. If \code{TRUE} (default) extracts converted counts,
otherwise returns original counts.}
}
\value{
A \code{data.frame}.
}
\description{
From the output of the function \code{\link[=format_data]{format_data()}} (or \code{\link[=filter_series]{filter_series()}}), this
function extracts \code{data.frame} containing converted counts
(\code{converted = TRUE}) or original counts (\code{converted = FALSE}) for one,
several, or all count series.

The resulting \code{data.frame} has no particular use in \code{popbayes} but it can be
useful for users.
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")

## Extract converted count data ----
converted_data <- popbayes::series_to_df(garamba_formatted, 
                                         converted = TRUE)

## Extract original count data ----
original_data <- popbayes::series_to_df(garamba_formatted, 
                                        converted = FALSE)

dim(converted_data)
dim(original_data)
dim(garamba)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bugs-funs.R
\name{bugs_to_df}
\alias{bugs_to_df}
\title{Extract estimated parameters from a list of BUGS outputs}
\usage{
bugs_to_df(data)
}
\arguments{
\item{data}{a named \code{list} of BUGS outputs. The output of \code{\link[=fit_trend]{fit_trend()}} or
\code{\link[=read_bugs]{read_bugs()}}}
}
\value{
A \code{data.frame}.
}
\description{
From the output of the function \code{\link[=fit_trend]{fit_trend()}} (or \code{\link[=read_bugs]{read_bugs()}}), this
function extracts estimated parameters into a \code{data.frame}.

The resulting \code{data.frame} has no particular use in \code{popbayes} but it can be
useful for users.
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")
                                        
## Select one serie ----
a_buselaphus <- popbayes::filter_series(garamba_formatted, 
                                        location = "Garamba",
                                        species  = "Alcelaphus buselaphus")
\donttest{
## Fit population trends (requires JAGS) ----
a_buselaphus_mod <- popbayes::fit_trend(a_buselaphus, path = temp_path)

## Import BUGS outputs for one count series ----
bugs <- popbayes::read_bugs(series = "garamba__alcelaphus_buselaphus", 
                            path   = temp_path)

## Extract estimated parameters ----
popbayes::bugs_to_df(bugs)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{garamba}
\alias{garamba}
\title{African mammals survey in the Garamba National Park}
\format{
A \code{data.frame} with 141 rows (counts) and the following 8 variables:
\describe{
\item{location}{the location of the survey (Garamba)}
\item{species}{the binomial name of the species}
\item{date}{the date of the survey}
\item{stat_method}{the method used to estimate individuals counts. One of
\code{T} (total counts), \code{G} (guesstimate), and \code{S}
(sampling counts)}
\item{field_method}{the field method used to collect data. One of
\code{A} (aerial counts), and \code{G} (ground counts)}
\item{count}{number of individuals}
\item{lower_ci}{lower boundary of the 95\% confidence interval of the counts
(only for sampling counts)}
\item{upper_ci}{upper boundary of the 95\% confidence interval of the counts
(only for sampling counts)}
\item{pref_field_method}{the preferred field method of the species. One of
\code{A} for Aerial counts, and \code{G} for Ground counts}
\item{conversion_A2G}{the conversion multiplicative factor (corresponding
to the detectability category) used to convert aerial to ground counts}
\item{rmax}{the maximum population growth rate}
}
}
\usage{
garamba
}
\description{
This dataset contains individual counts of 10 African mammal species
in the Garamba National Park (Democratic Republic of the Congo) from 1976 to
2017.
}
\examples{
data("garamba")
head(garamba, 30)
}
\references{
Hillman Smith K & Kalpers J (2015) \emph{Garamba Conservation in Peace & War}.
Hillman Smith publisher, 448pp. ISBN: 9789966185105.\cr
Monico M (2014)
\emph{Aerial Survey Report March 2014 - Garamba National Park, DRC}.
African Parks Network/ICCN/Pan-African Elephant Aerial Survey, 42pp.\cr
Spies K \emph{et al.} (2017)
\emph{Aerial Survey Report April 2017 - Garamba National Park, DRC}.
African Parks Network/EU/ICCN. 38pp.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{species_info}
\alias{species_info}
\title{Species information dataset}
\format{
A \code{data.frame} with 15 rows (African mammals species) and the
following variables:
\describe{
\item{order}{the order of the species}
\item{family}{the family of the species}
\item{species}{the species binomial name}
\item{english}{the species English name}
\item{french}{the species French name}
\item{category}{the detectability category of the species. One of
\code{MLB} for Medium-sized Light and Brown species (20-150kg),
\code{LLB} for Large Light and Brown species (>150kg),
\code{LD} for Large Dark (>150kg), \code{Elephant}, and \code{Giraffe}}
\item{pref_field_method}{the preferred field method of the species. One of
\code{A} for Aerial counts, and \code{G} for Ground counts}
\item{conversion_A2G}{the conversion multiplicative factor (corresponding
to the detectability category) used to convert aerial to ground counts}
\item{rmax}{the maximum population growth rate}
}
}
\usage{
species_info
}
\description{
This dataset contains information about 15 African mammal species.
It can be used in the function \code{format_data()} to convert individual counts
estimated from a field method to a preferred field method. The field method
can be \code{A} (aerial counts) or \code{G} (ground counts). See \code{\link[=format_data]{format_data()}} for
further information. It also contains the maximum population growth rate
(i.e. the maximum change in log population size).

User can take this dataset as a template to add information for missing
species. Note that only \code{species}, \code{pref_field_method}, \code{conversion_A2G},
and \code{rmax} are required.
}
\examples{
data("species_info")
species_info
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/series-funs.R
\name{read_series}
\alias{read_series}
\title{Import a list of count series previously exported}
\usage{
read_series(series = NULL, path = ".")
}
\arguments{
\item{series}{a vector of \code{character} strings. One or several count series
names to be imported. If \code{NULL} (default), all available count series
will be imported.}

\item{path}{a \code{character} string. The directory in which count series have
been saved by the function \code{\link[=format_data]{format_data()}}.}
}
\value{
An n-element \code{list} (where \code{n} is the number of count series). See
\code{\link[=format_data]{format_data()}} for further information.
}
\description{
This function imports a list of count series data previously exported by
\code{\link[=format_data]{format_data()}}. Users can import one, several, or all count series data.
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")

## Import all count series ----
count_series <- popbayes::read_series(path = temp_path)

## Import one count series ----
a_bus <- popbayes::read_series(series = "garamba__alcelaphus_buselaphus",
                               path   = temp_path)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/series-funs.R
\name{filter_series}
\alias{filter_series}
\title{Extract the count series corresponding to a location and/or a species}
\usage{
filter_series(data, species = NULL, location = NULL)
}
\arguments{
\item{data}{a named \code{list}. The output of function \code{\link[=format_data]{format_data()}}.}

\item{species}{a \code{character} string. A species name.}

\item{location}{a \code{character} string. A site name.}
}
\value{
A subset of \code{data}, i.e. a named \code{list}.
}
\description{
This function identifies the count series relative to a species and/or
a location in a named list like the output of function \code{\link[=format_data]{format_data()}}. If
both species and location are provided, the series of counts of the species
at the specified location is extracted. Otherwise, all series corresponding
to the specified criterion (species or location) are extracted.
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")

## Number of count series ----
length(garamba_formatted)

## Retrieve count series names ----
popbayes::list_series(path = temp_path)

## Get data for Alcelaphus buselaphus (at all sites) ----
x <- popbayes::filter_series(garamba_formatted, 
                             species = "Alcelaphus buselaphus")

## Get data at Garamba (for all species) ----
x <- popbayes::filter_series(garamba_formatted, 
                             location = "Garamba")

## Get data for Alcelaphus buselaphus at Garamba only ----
x <- popbayes::filter_series(garamba_formatted, 
                             location = "Garamba",
                             species  = "Alcelaphus buselaphus")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bugs-funs.R
\name{read_bugs}
\alias{read_bugs}
\title{Import a list of BUGS outputs previously exported}
\usage{
read_bugs(series = NULL, path = ".")
}
\arguments{
\item{series}{a vector of \code{character} strings. One or several count series
names. If \code{NULL} (default) BUGS outputs for all count series will be
imported. Users can run \code{\link[=list_series]{list_series()}} to get the correct spelling of
count series names.}

\item{path}{a \code{character} string. The directory in which BUGS outputs have
been saved by the function \code{\link[=fit_trend]{fit_trend()}}.}
}
\value{
An n-element \code{list} (where \code{n} is the number of count series). See
\code{\link[=fit_trend]{fit_trend()}} for further information.
}
\description{
This function imports a list of BUGS outputs previously exported by
\code{\link[=fit_trend]{fit_trend()}}. Users can import one, several, or all models.
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")

## Select one serie ----
a_buselaphus <- popbayes::filter_series(garamba_formatted, 
                                        location = "Garamba",
                                        species  = "Alcelaphus buselaphus")
\donttest{
## Fit population trends (requires JAGS) ----
a_buselaphus_mod <- popbayes::fit_trend(a_buselaphus, path = temp_path)

## Import BUGS outputs for one count series ----
popbayes::read_bugs(series = "garamba__alcelaphus_buselaphus", 
                    path   = temp_path)

## Import BUGS outputs for all count series ----
popbayes::read_bugs(path = temp_path)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{w_to_rmax}
\alias{w_to_rmax}
\title{Compute rmax from adult female body mass}
\usage{
w_to_rmax(w)
}
\arguments{
\item{w}{a numerical vector. Adult female body mass (in kg).}
}
\value{
A numerical vector of \code{rmax} values.
}
\description{
The demographic potential of a species is limited. The intrinsic rate of
increase \code{rmax} is the maximum increase in log population size that a
species can attain in a year. According to Sinclair (2003), it is related
to the body mass of adult females by: \eqn{1.375 \times W^{-0.315}}
}
\examples{
## Set adult female body mass ----
body_masses <- c(55, 127)

## Add species names ----
names(body_masses) <- c("Impala", "Tiang")

## Compute species rmax ----
w_to_rmax(body_masses)
}
\references{
Sinclair (2013) Mammal population regulation, keystone processes and
ecosystem dynamics. \emph{Philosophical Transactions: Biological Sciences},
\strong{358}, 1729-1740.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format-funs.R
\name{format_data}
\alias{format_data}
\title{Format count series}
\usage{
format_data(
  data,
  info = NULL,
  date = "date",
  count = "count",
  location = "location",
  species = "species",
  stat_method = "stat_method",
  lower_ci = "lower_ci",
  upper_ci = "upper_ci",
  sd = NULL,
  var = NULL,
  cv = NULL,
  field_method = NULL,
  pref_field_method = NULL,
  conversion_A2G = NULL,
  rmax = NULL,
  path = ".",
  na_rm = FALSE
)
}
\arguments{
\item{data}{a \code{data.frame} with at least five columns: \code{location},
\code{species}, \code{date}, \code{count}, and \code{stat_method}.

The \code{stat_method} field indicates the method used to estimate counts. It
can contain: \code{T} (total counts), \code{X} (guesstimate), and/or \code{S} (sampling).

If individual counts were estimated by \strong{sampling}, additional column(s)
providing a measure of precision is also required (e.g. \code{lower_ci} and
\code{upper_ci}, or \code{sd}, \code{cv}, \code{var}). Precision metrics can be different
between counts. For instance, some sampling counts can have a \code{sd} value
and others \code{lower_ci} and \code{upper_ci}. In that case three columns are
required (\code{lower_ci}, \code{upper_ci}, and \code{sd}). See above section
\strong{Description} for further information on the computation of the 95\%
confident interval of estimates.

If the individuals were counted by different methods, an additional field
\code{field_method} is also required. It can contain: \code{G} (ground counts)
and/or \code{A} (aerial counts). See above section \strong{Description} for further
information on the counts conversion.

Others fields can be present either in \code{data} or \code{info} (see below).}

\item{info}{(optional) a \code{data.frame} with species in rows and the following
columns: \code{species} (species name), \code{pref_field_method},
\code{conversion_A2G}, and \code{rmax}. See above section \strong{Description} for
further information on these fields.
Default is \code{NULL} (i.e. these information must be present in \code{data}
if not available in \code{popbayes}).}

\item{date}{a \code{character} string. The column name in \code{data} of the date.
This column \code{date} must be in a numerical form with possibly a decimal
part.
Default is \code{'date'}.}

\item{count}{a \code{character} string. The column name in \code{data} of the
number of individuals. This column must be numerical.
Default is \code{'count'}.}

\item{location}{a \code{character} string. The column name in \code{data} of the
site. This field is used to distinguish count series from different sites
(if required) and to create an unique series name.
Default is \code{'location'}.}

\item{species}{a \code{character} string. The column name in \code{data} (and
in \code{info} if provided) of the species. This field is used to distinguish
count series for different species (if required) and to create an unique
series name.
Default is \code{'species'}.}

\item{stat_method}{a \code{character} string. The column name in \code{data} of
the method used to estimate individuals counts. It can contain \code{'T'}
(total counts), \code{'X'} (guesstimate), and/or \code{'S'} (sampling). If some
counts are coded as \code{'S'}, precision column(s) must also be provided (see
below).
Default is \code{'stat_method'}.}

\item{lower_ci}{(optional) a \code{character} string. The column name in \code{data}
of the lower boundary of the 95\% CI of the estimate (i.e. \code{count}). If
provided, the upper boundary of the 95\% CI (argument \code{upper_ci}) must be
also provided. This argument is only required if some counts have been
estimated by a sampling method. But user may prefer use other precision
measures, e.g. standard deviation (argument \code{sd}), variance (argument
\code{var}), or coefficient of variation (argument \code{cv}).
Default is \code{'lower_ci'}.}

\item{upper_ci}{(optional) a \code{character} string. The column name in \code{data}
of the upper boundary of the 95\% CI of the estimate (i.e. \code{count}). If
provided, the lower boundary of the 95\% CI (argument \code{lower_ci}) must be
also provided.
Default is \code{'upper_ci'}.}

\item{sd}{(optional) a \code{character} string. The column name in \code{data} of
the standard deviation of the estimate.
Default is \code{NULL}.}

\item{var}{(optional) a \code{character} string. The column name in \code{data} of
the variance of the estimate.
Default is \code{NULL}.}

\item{cv}{(optional) a \code{character} string. The column name in \code{data} of
the coefficient of variation of the estimate.
Default is \code{NULL}.}

\item{field_method}{(optional) a \code{character} string. The column name in
\code{data} of the field method used to count individuals. Counts can be ground
counts (coded as \code{'G'}) or aerial counts (coded as \code{'A'}). This argument
is optional if individuals have been counted by the same method. See above
section \strong{Description} for further information on the count conversion.
Default is \code{NULL}.}

\item{pref_field_method}{(optional) a \code{character} string. The column name
in \code{data} of the preferred field method of the species. This argument is
only required is \code{field_method} is not \code{NULL} (i.e. individuals have been
counted by different methods). Alternatively, this value can be passed in
\code{info} (or internally retrieved if the species is listed in the package).
See above section \strong{Description} for further information on the count
conversion.
Default is \code{NULL}.}

\item{conversion_A2G}{(optional) a \code{character} string. The column name
in \code{data} of the count conversion factor of the species. This argument is
only required if \code{field_method} is not \code{NULL} (i.e. individuals have been
counted by different methods). Alternatively this value can be passed in
\code{info} (or internally retrieved if the species is listed in the package).
See above section \strong{Description} for further information on the count
conversion.
Default is \code{NULL}.}

\item{rmax}{(optional) a \code{character} string. The column name in \code{data} of
the species demographic potential (i.e. the relative rate of increase of
the population). This is the change in log population size between two
dates and will be used later by \code{\link[=fit_trend]{fit_trend()}}.
Default is \code{NULL}.}

\item{path}{a \code{character} string. The directory to save formatted data.
This directory must exist and can be an absolute or a relative path.
Default is the current working directory.}

\item{na_rm}{a \code{logical.} If \code{TRUE}, counts with \code{NA} values will be
removed.
Default is \code{FALSE} (returns an error to inform user if \code{NA} are detected).}
}
\value{
An n-elements \code{list} (where \code{n} is the number of count series). The
name of each element of this list is a combination of location and
species. Each element of the list is a \code{list} with the following content:
\itemize{
\item \code{location} a \code{character} string. The name of the series site.
\item \code{species} a \code{character} string. The name of the series species.
\item \code{date} a \code{numerical} vector. The sequence of dates of the
series.
\item \code{n_dates} an \code{integer.} The number of unique dates.
\item \code{stat_methods} a \code{character} vector. The different stat methods
of the series.
\item \code{field_methods} (optional) a \code{character} vector. The different
field methods of the series.
\item \code{pref_field_method} (optional) a \code{character} string. The
preferred field method of the species (\code{'A'} or \code{'G'}).
\item \code{conversion_A2G} (optional) a \code{numeric}. The conversion factor
of the species used to convert counts to its preferred field method.
\item \code{rmax} a \code{numeric}. The maximum population growth rate of the
species.
\item \code{data_original} a \code{data.frame}. Original data of the series
with renamed columns. Some rows may have been deleted
(if \code{na_rm = TRUE}).
\item \code{data_converted} a \code{data.frame}. Data containing computed
boundaries of the 95\% CI (\code{lower_ci_conv} and \code{upper_ci_conv}). If
counts have been obtained by different field methods, contains also
converted counts (\code{count_conv}) based on the preferred field method and
conversion factor of the species. This \code{data.frame} will be used by the
function \code{\link[=fit_trend]{fit_trend()}} to fit population models.
}

\strong{Note:} Some original series can be discarded if one of these two
conditions is met: 1) the series contains only zero counts, and 2) the
series contains only a few dates (< 4 dates).
}
\description{
This function provides an easy way to get count series ready to be analyzed
by the package \code{popbayes}. It must be used prior to all other functions.

This function formats the count series (passed through the argument
\code{data}) by selecting and renaming columns, checking columns format and
content, and removing missing data (if \code{na_rm = TRUE}). It converts the
original data frame into a list of count series that will be analyzed later
by the function \code{\link[=fit_trend]{fit_trend()}} to estimate population trends.

To be usable for the estimation of population trends, counts must be
accompanied by information on precision. The population trend model requires
a 95\% confident interval (CI).
If estimates are total counts or guesstimates, this function will construct
boundaries of the 95\% CI by applying the rules set out in
\url{https://frbcesab.github.io/popbayes/articles/popbayes.html}.
If counts were estimated by a sampling method the user needs to specify a
measure of precision. Precision is preferably provided in the form of a 95\%
CI by means of two fields: \code{lower_ci} and \code{upper_ci}. It may also be given
in the form of a standard deviation (\code{sd}), a variance (\code{var}), or a
coefficient of variation (\code{cv}). If the fields \code{lower_ci} and \code{upper_ci} are
both absent (or \code{NA}), fields \code{sd}, \code{var}, and \code{cv} are examined in this
order. When one is found valid (no missing value), a 95\% CI is derived
assuming a normal distribution.
The field \code{stat_method} must be present in \code{data} to indicate
if counts are \strong{total counts} (\code{'T'}), \strong{sampling} (\code{'S'}), or
\strong{guesstimate} (\code{'X'}).

If a series mixes aerial and ground counts, a field \code{field_method} must
also be present and must contain either \code{'A'} (aerial counts), or \code{'G'}
(ground counts). As all counts must eventually refer to the same field
method for a correct estimation of trend, a conversion will be performed to
homogenize counts. This conversion is based on a \strong{preferred field method}
and a \strong{conversion factor} both specific to a species/category.
The preferred field method specifies the conversion direction. The
conversion factor is the multiplicative factor that must be applied to an
aerial count to get an equivalent ground count (note that if the preferred
field method is \code{'A'}, ground counts will be divided by the conversion
factor to get the equivalent aerial count).

The argument \code{rmax} represents the maximum change in log population size
between two dates (i.e. the relative rate of increase). It will be used
by \code{\link[=fit_trend]{fit_trend()}} but must be provided in this function.

These three parameters, named \code{pref_field_method}, \code{conversion_A2G}, and
\code{rmax} can be present in \code{data} or in a second \code{data.frame}
(passed through the argument \code{info}).
Alternatively, the package \code{popbayes} provides their values for some
African large mammals.

\strong{Note:} If the field \code{field_method} is absent in \code{data}, counts are
assumed to be obtained with one field method.
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")

## Number of count series ----
length(garamba_formatted)

## Retrieve count series names ----
popbayes::list_series(path = temp_path)

## Print content of the first count series ----
names(garamba_formatted[[1]])

## Print original data ----
garamba_formatted[[1]]$"data_original"

## Print converted data ----
garamba_formatted[[1]]$"data_converted"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-funs.R
\name{plot_trend}
\alias{plot_trend}
\title{Plot estimated population trend}
\usage{
plot_trend(series, title = TRUE, path = ".", path_fig = ".", save = FALSE)
}
\arguments{
\item{series}{a \code{character} string. The count series name (can be
retrieved by running \code{\link[=list_series]{list_series()}}).}

\item{title}{a \code{logical}. If \code{TRUE} (default) a title (series name) is
added.}

\item{path}{a \code{character} string. The directory in which count series (and
BUGS outputs) have been saved by the function \code{\link[=format_data]{format_data()}} (and by
\code{\link[=fit_trend]{fit_trend()}}).}

\item{path_fig}{a \code{character} string. The directory where to save the plot
(if \code{save = TRUE}). This directory must exist and can be an absolute or a
relative path.}

\item{save}{a \code{logical}. If \code{TRUE} (default is \code{FALSE}) the plot is saved in
\code{path_fig}.}
}
\value{
No return value.
}
\description{
This function plots a panel of two graphics for one BUGS model
(previously generated by \code{\link[=fit_trend]{fit_trend()}}):
\itemize{
\item on the left side, the population trend estimated by the Bayesian model
(blue line) with the 95\% CI (gray envelop). Dots (with intervals)
represent converted counts passed to the model (with the 95\% CI);
\item on the right side, a bar plot of estimated relative growth rates (r) by
date. Dark bars are real estimated r.
}
}
\examples{
## Load Garamba raw dataset ----
file_path <- system.file("extdata", "garamba_survey.csv", 
                         package = "popbayes")
                         
garamba <- read.csv(file = file_path)

## Create temporary folder ----
temp_path <- tempdir()

## Format dataset ----
garamba_formatted <- popbayes::format_data(
  data              = garamba, 
  path              = temp_path,
  field_method      = "field_method",
  pref_field_method = "pref_field_method",
  conversion_A2G    = "conversion_A2G",
  rmax              = "rmax")

## Select one serie ----
a_buselaphus <- popbayes::filter_series(garamba_formatted, 
                                        location = "Garamba",
                                        species  = "Alcelaphus buselaphus")
\donttest{
## Fit population trends (requires JAGS) ----
a_buselaphus_mod <- popbayes::fit_trend(a_buselaphus, path = temp_path)

## Plot estimated population trend ----
popbayes::plot_trend(series = "garamba__alcelaphus_buselaphus", 
                     path   = temp_path)
}
}
